#-------------------------------------------------------------------------------
#
# Copyright (c) 2009, IMB, RWTH Aachen.
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in simvisage/LICENSE.txt and may be redistributed only
# under the conditions described in the aforementioned license.  The license
# is also available online at http://www.simvisage.com/licenses/BSD.txt
#
# Thanks for using Simvisage open source!
#
# Created on May 12, 2010 by: rch

from traits.api import \
    HasTraits, List, Float, \
    on_trait_change, Instance, \
    Str, Property, cached_property, \
    Event, \
    DelegatesTo, Button

from traitsui.api import \
    View, Item, DirectoryEditor, TabularEditor, HSplit, VGroup, \
    TableEditor, EnumEditor, Handler, FileEditor, VSplit, Group, \
    InstanceEditor, HGroup, Spring

from traitsui.tabular_adapter \
    import TabularAdapter

from traitsui.table_column import \
    ObjectColumn

# overload the 'get_label' method from 'Item' to display units in the label
from util.traits.ui.item import \
    Item

from bmcs_beam.mxn.mfn.mfn_line.mfn_line import \
    MFnLineArray

from matplotlib.figure import \
    Figure

from util.traits.editors.mpl_figure_editor import \
    MPLFigureEditor

from numpy import \
    array, ones_like, frompyfunc, linspace

import numpy as np

from .concrete_mixture \
    import ConcreteMixture

from .fabric_layout \
    import FabricLayOut

from .fabric_layup \
    import FabricLayUp

from matresdev.db.simdb.simdb_class import \
    SimDBClass, SimDBClassExt

class DamageFunctionEntry(HasTraits):
    material_model = Str(input=True)
    calibration_test = Str(input=True)
    damage_function = Instance(MFnLineArray, input=True)

    input_change = Event
    @on_trait_change('+input')
    def _set_input_change(self):
        self.input_change = True


df_table_editor = TableEditor(
                        columns=[ ObjectColumn(name='material_model',
                                                        editable=False,
                                                        horizontal_alignment='center'),
                                          ObjectColumn(name='calibration_test',
                                                        editable=False,
                                                        horizontal_alignment='center') ],
                        selection_mode='rows',
                        selected='object.selected_dfs',
                        deletable=True,
                        editable=False,
                        show_toolbar=True,
                        auto_add=False,
                        configurable=False,
                        sortable=False,
                        reorderable=False,
                        sort_model=False,
                        orientation='vertical',
                        auto_size=True,
            )

ccsuc_table_editor = TableEditor(
                        columns=[ ObjectColumn(name='fabric_layout_key',
                                                  editable=False, width=0.20),
                                    ObjectColumn(name='concrete_mixture_key',
                                                  editable=False, width=0.20),
                                    ObjectColumn(name='s_tex_z',
                                                  editable=False, width=0.20),
                                    ],
                        selection_mode='row',
                        selected='object.selected_ccuc',
                        show_toolbar=True,
                        auto_add=False,
                        configurable=True,
                        sortable=True,
                        deletable=False,
                        reorderable=True,
                        sort_model=False,
                        auto_size=False,
#                        filters      = [EvalFilterTemplate,
#                                       MenuFilterTemplate,
#                                       RuleFilterTemplate ],
#                        search       = EvalTableFilter(),
            )

class CCSUnitCell(SimDBClass):
    '''Describes the combination of the concrete mixture and textile cross section.
    Note: only one concrete can be defined for the entire composite cross section.
    '''

    input_change = Event
    @on_trait_change('+input, damage_function_list.input_change')
    def _set_input_change(self):
        self.input_change = True

    #--------------------------------------------------------------------------------
    # select the concrete mixture from the concrete database:
    #--------------------------------------------------------------------------------

    concrete_mixture_key = Str(simdb=True, input=True, table_field=True,
                                auto_set=False, enter_set=True)

    concrete_mixture_ref = Property(Instance(SimDBClass),
                                     depends_on='concrete_mixture_key')
    @cached_property
    def _get_concrete_mixture_ref(self):
        return ConcreteMixture.db[ self.concrete_mixture_key ]

    #--------------------------------------------------------------------
    # textile fabric layout used
    #--------------------------------------------------------------------
    #
    fabric_layout_key = Str(simdb=True, input=True, table_field=True,
                             auto_set=False, enter_set=True)
    def _fabric_layout_key_default(self):
        return list(FabricLayOut.db.keys())[0]

    fabric_layout_ref = Property(Instance(SimDBClass),
                                  depends_on='fabric_layout_key')
    @cached_property
    def _get_fabric_layout_ref(self):
        return FabricLayOut.db[ self.fabric_layout_key ]

    # equidistant spacing in thickness direction (z-direction) between the layers
    #
    s_tex_z = Float(1, unit='m', simdb=True, input=True, table_field=True,
                      auto_set=False, enter_set=True)

    # orientation function of the layup fabric
    #
    orientation_fn_key = Str('', simdb=True, table_field=True,
                                auto_set=False, enter_set=True)

    #--------------------------------------------------------------------------------
    # Derived material parameters - supplied during the calibration
    #--------------------------------------------------------------------------------

    def get_E_m_time(self, age):
        '''function for the composite E-modulus as weighted sum of all
        fabric layups. Returns a function depending of the concrete age.
        '''
        E_m = self.concrete_mixture_ref.get_E_m_time(age)
        print('E_m_time', E_m)
        return E_m

    def get_E_c_time(self, age):
        '''function for the composite E-modulus as weighted sum of all
        fabric layups. Returns a function depending of the concrete age.
        '''
        rho = self.rho
        E_tex = self.E_tex
        E_m = self.concrete_mixture_ref.get_E_m_time(age)
        E_c = (1 - rho) * E_m + rho * E_tex
        return E_c

    E_c28 = Property(Float, unit='MPa', depends_on='input_change')
    @cached_property
    def _get_E_c28(self):
        '''Composite E-modulus after 28 days.
        '''
        return self.get_E_c_time(28)

    nu = DelegatesTo('concrete_mixture_ref', listenable=False)

    #--------------------------------------------------------------------------------
    # Defined material parameters - supplied during the calibration
    #--------------------------------------------------------------------------------
    rho = Float

    E_tex = Property(depends_on='input_changed')
    def _get_E_tex(self):
        return self.fabric_layout_ref.E_tex_0

    #--------------------------------------------------------------------------------
    # Bag for material parameters
    #--------------------------------------------------------------------------------
    #
    # The material parameters are quantities to be used by models that should make
    # prognosis of the behavior in general condition. The composite cross section
    # can be modeled by a variety of models, each of which may need different
    # parameters. The parameters are determined using some calibration test.
    # Therefore, in order to trace back the origin of the material parameters
    # it is necessary to store the value of the material parameter together with
    # the model that has calibrated it and by the test setup that was used
    # for calibration. In this way, the range of validity of the calibrated damage
    # function is available.
    #
    # The material parameters are stored in a list with the combined key
    # containing the tuple (material model, test run).
    #
    damage_function_list = List(DamageFunctionEntry, input=True)

    def set_param(self, material_model, calibration_test, df):
        print('adding damage function')
        for mp_entry in self.damage_function_list:
            if (mp_entry.material_model == material_model and
                 mp_entry.calibration_test == calibration_test):
                mp_entry.damage_function = df
                print('only the function updated')
                return
        mp_entry = DamageFunctionEntry(material_model=material_model,
                                        calibration_test=calibration_test,
                                        damage_function=df)
        self.damage_function_list.append(mp_entry)
        print('new function added')

    def get_param(self, material_model, calibration_test):
        print('material_model', material_model)
        print('calibration_test', calibration_test)
        for mp_entry in self.damage_function_list:
            if (mp_entry.material_model == material_model and
                 mp_entry.calibration_test == calibration_test):
                return mp_entry.damage_function

        raise ValueError('no entry in unit cell with key ( %s ) for model ( %s ) and test( %s )' % (self.key, material_model, calibration_test))

    #-------------------------------------------------------------
    # derive the average phi-function based on all entries
    # in damage_function_list
    #-------------------------------------------------------------

    df_average = Property(Instance(MFnLineArray),
                           depends_on='damage_function_list')
    @cached_property
    def get_df_average(self, n_points):
        '''derive the average phi-function based on all entries
        in damage_function_list
        '''

        def get_y_average(self, x_average):
            '''get the y-values from the mfn-functions in df_list for
            'x_average' and return the average.
            Note that the shape of 'mfn.xdata' does not necessarily needs to be equal in all
            'DamageFunctionEntries' as the number of steps used for calibration or the adaptive
            refinement in 'tloop' might have been different for each case.
            '''
            y_list = [ self.damage_function_list[i].damage_function.get_value(x_average) \
                       for i in range(len(self.damage_function_list)) ]
            return sum(y_list) / len(y_list)

        get_y_average_vectorized = frompyfunc(get_y_average, 2, 1)

        mfn = MFnLineArray()

        # take the smallest value of the strains for the average function. Beyond this value
        # the average does not make sense anymore because it depends on the arbitrary number
        # of entries in the df_list
        #
        xdata_min = min(self.damage_function_list[i].damage_function.xdata[-1] \
                         for i in range(len(self.damage_function_list)))

        # number of sampling point used for the average phi function
        #
        mfn.xdata = linspace(0., xdata_min, num=n_points)

        # get the corresponding average ydata values
        #
        mfn.ydata = self.get_y_average_vectorized(mfn.xdata)

        return mfn


    #-------------------------------------------------------------------


    #-------------------------------------------------------------------
    # PLOT OBJECT
    #-------------------------------------------------------------------

    figure = Instance(Figure, transient=True)
    def _figure_default(self):
        figure = Figure(facecolor='white')
        figure.add_axes([0.08, 0.13, 0.85, 0.74])
        return figure

    selected_dfs = List(Instance(DamageFunctionEntry), transient=True)
    def _selected_dfs_default(self):
        return self.damage_function_list

    print_values = Button()
    def _print_values_fired(self):
        for mp_entry in self.selected_dfs:
            print('x', mp_entry.damage_function.xdata)
            print('y', mp_entry.damage_function.ydata)

    save_phi_fn_values = Button()
    def _save_phi_fn_values_fired(self):
        'save phi_fn xy-data to file'
        print('save phi_arr data to file')
        for mp_entry in self.selected_dfs:
            xdata = mp_entry.damage_function.xdata
            ydata = mp_entry.damage_function.ydata
            phi_arr = np.hstack([xdata[:, None], ydata[:, None]])
            print('phi_arr ', phi_arr.shape, ' save to file "phi_arr.csv"')
            np.savetxt('phi_arr.csv', phi_arr, delimiter=';')

    # event to trigger the replotting - used by the figure editor
    #
    data_changed = Event

#    @on_trait_change('damage_function_list')
    @on_trait_change('input_change, selected_dfs')
    def _redraw(self):
        # map the array dimensions to the plot axes
        #
        figure = self.figure
        axes = figure.gca()
        axes.clear()

        for mp_entry in self.selected_dfs:
#        for mp_entry in self.damage_function_list:
            df = mp_entry.damage_function
            mp_entry.damage_function.mpl_plot(axes)

        self.data_changed = True

    #--------------------------------------------------------------------------------
    # view
    #--------------------------------------------------------------------------------

    traits_view = View(Item('key', style='readonly'),
                       Item('s_tex_z', style='readonly', format_str="%.5f"),
                        Item('orientation_fn_key', style='readonly'),
                        Item('rho', style='readonly', format_str="%.5f"),
                        VSplit(
                        VGroup(
                            Spring(),
                            Item('concrete_mixture_key', style='readonly',
                                label='concrete mixture'),
                            Spring(),
                            Item('concrete_mixture_ref@', style='readonly',
                                 show_label=False),
                            Spring(),
                            id='exdb.ccsuc.cm',
                            label='concrete mixture',
                            dock='tab',
                            ),
                        VGroup(
                            Spring(),
                            Item('fabric_layout_key', style='readonly',
                                label='fabric layout'),
                            Spring(),
                            Item('fabric_layout_ref@', style='readonly',
                                show_label=False),
                            Spring(),
                            id='exdb.ccsuc.lo',
                            label='fabric layout',
                            dock='tab',
                            ),
                        Group(
                              Item('print_values', show_label=False),
                              Item('save_phi_fn_values', show_label=False),
                        Item('damage_function_list@', editor=df_table_editor,
                             resizable=True, show_label=False),
                             id='exdb.ccsuc.damage_functions',
                             label='damage functions',
                             dock='tab',
                        ),
                        Group(
                        Item('figure', editor=MPLFigureEditor(),
                             resizable=True, show_label=False),
                             id='exdb.ccsuc.plot_sheet',
                             label='plot sheet',
                             dock='tab',

                        ),
#                        Group(
#                        Item('rho_c',
#                             style = 'readonly', show_label = True, format_str="%.4f"),
#                        Item('E_c28',
#                             style = 'readonly', show_label = True, format_str="%.0f" ),
#                        label = 'derived params',
#                        id = 'exdb.ccsuc.dp',
#                        dock = 'tab',
#                        ),

                        dock='tab',
                        id='exdb.ccsuc.db.vsplit',
                        orientation='vertical',
                        ),
                        dock='tab',
                        id='ccsuc.db',
                        scrollable=True,
                        resizable=True,
                        height=0.4,
                        width=0.5,
                        buttons=['OK', 'Cancel'],
                        )

# Setup the database class extension
#
CCSUnitCell.db = SimDBClassExt(
            klass=CCSUnitCell,
            verbose='io',
            )

if __name__ == '__main__':

    CCSUnitCell.db.configure_traits()



'''
Created on 31. 1. 2014

@author: Vancikv
'''

from traits.api import \
    Float, Property, cached_property, \
    Button

from bmcs_beam.mxn.material_types import \
    MTReinfFabric

from traitsui.api import \
    View, Item, VGroup, Group

from .reinf_layout_component import \
    ReinfLayoutComponent, \
    STATE_LAW_AND_GEOMETRY_CHANGE, \
    STATE_AND_GEOMETRY_CHANGE

import numpy as np

from bmcs_beam.mxn.utils import \
    KeyRef

from .reinf_fabric_handler import \
    FabricHandler

class RLCTexLayer(ReinfLayoutComponent):
    '''single layer of textile reinforcement
    '''
    def __init__(self, *args, **metadata):
        if not metadata.get('material', None):
            metadata['material'] = 'default_fabric'
        super(RLCTexLayer, self).__init__(**metadata)

    z_coord = Float(0.2, auto_set=False, enter_set=True, geo_input=True)
    '''distance of the layer from the bottom'''

    material = KeyRef('default_fabric', db=MTReinfFabric.db, law_input=True)

    def convert_eps_u_2_lo(self, eps_up):
        h = self.matrix_cs.geo.height
        eps_u = self.material_law_.eps_u
        eps_lo = eps_up + (eps_u - eps_up) / (h - self.z_coord) * h
        return eps_lo

    def convert_eps_u_2_up(self, eps_lo):
        h = self.matrix_cs.geo.height
        eps_u = self.material_law_.eps_u
        eps_up = eps_lo + (eps_u - eps_lo) / self.z_coord * h
        return eps_up

    #===========================================================================
    # Discretization conform to the tex layers
    #===========================================================================

    n_rovings = Property(depends_on='material,material_changed,matrix_cs.geo.changed')
    '''Number of rovings in the textile layer
    '''
    @cached_property
    def _get_n_rovings(self):
        return int(self.matrix_cs.geo.get_width(self.z_coord) / self.material_.s_0) - 1

    eps = Property(depends_on=STATE_AND_GEOMETRY_CHANGE)
    '''Strain at the level of the reinforcement layer
    '''
    @cached_property
    def _get_eps(self):
        # ------------------------------------------------------------------------
        # geometric params independent from the value for 'eps_t'
        # ------------------------------------------------------------------------
        height = self.matrix_cs.geo.height
        eps_lo = self.state.eps_lo
        eps_up = self.state.eps_up
        # strain at the height of each reinforcement layer [-]:
        #
        return eps_lo + (eps_up - eps_lo) * self.z_coord / height

    eps_t = Property(depends_on=STATE_AND_GEOMETRY_CHANGE)
    '''Tension strain at the level of the layer of the fabrics
    '''
    @cached_property
    def _get_eps_t(self):
        return (np.fabs(self.eps) + self.eps) / 2.0

    eps_c = Property(depends_on=STATE_AND_GEOMETRY_CHANGE)
    '''Compression strain at the level of the layer.
    '''
    @cached_property
    def _get_eps_c(self):
        return (-np.fabs(self.eps) + self.eps) / 2.0

    sig_t = Property(depends_on=STATE_LAW_AND_GEOMETRY_CHANGE)
    '''Stresses at the i-th fabric layer.
    '''
    @cached_property
    def _get_sig_t(self):
        return self.material_law_.mfn.get_value(self.eps_t)

    f_t = Property(depends_on=STATE_LAW_AND_GEOMETRY_CHANGE)
    '''force at the height of reinforcement layer [kN]:
    '''
    @cached_property
    def _get_f_t(self):
        sig_t = self.sig_t
        n_rovings = self.n_rovings
        A_roving = self.material_.A_roving
        return sig_t * n_rovings * A_roving / self.unit_conversion_factor

    N = Property(depends_on=STATE_LAW_AND_GEOMETRY_CHANGE)
    '''Get the resulting normal force.
    '''
    @cached_property
    def _get_N(self):
        return self.f_t

    M = Property(depends_on=STATE_LAW_AND_GEOMETRY_CHANGE)
    '''Get the resulting moment.
    '''
    @cached_property
    def _get_M(self):
        height = self.matrix_cs.geo.height
        return self.f_t * (height - self.z_coord)

    #===========================================================================
    # UI-related functionality
    #===========================================================================

    node_name = 'Textile layer'

    def plot_geometry(self, ax, clr='DarkOrange'):
        '''Plot geometry'''
        width = self.matrix_cs.geo.get_width(self.z_coord)
        w_max = self.matrix_cs.geo.width
        ax.hlines(self.z_coord, (w_max - width) / 2,
                  (w_max + width) / 2, lw=2, color=clr, linestyle='dashed')

    def plot_eps(self, ax):
        eps_lo = self.state.eps_lo
        eps_up = self.state.eps_up

        # eps t
        ax.hlines([self.z_coord], [0], [-self.eps_t], lw=4, color='DarkOrange')

        # reinforcement layer
        ax.hlines([self.z_coord], [min(0.0, -eps_lo, -eps_up)],
                  [max(0.0, -eps_lo, -eps_up)], lw=1, color='black', linestyle='--')

    def plot_sig(self, ax):
        # sig t
        ax.hlines([self.z_coord], [0], [-self.sig_t], lw=4, color='DarkOrange')

    tree_view = View(VGroup(
                      Group(
                      Item('z_coord'),
                      label='Geometry'
                      ),
                      Group(
                      Item('material'),
                      Item('material_law'),
                      label='Fabric material',
                      ),
                      springy=True,
                      ),
                resizable=True,
                handler=FabricHandler(),
                buttons=['OK', 'Cancel']
                )

if __name__ == '__main__':
    layer = RLCTexLayer()
    layer.fabric = 'default_fabric'
    layer.fabric_.A_roving = 0.5
    print(layer.fabric_.state_link_lst)
    layer.configure_traits()



'''
Created on Aug 23, 2012

@author: rch
'''

from bmcs_beam.mxn.matrix_laws import \
    MatrixLawBilinear, MatrixLawBlock, \
    MatrixLawLinear, MatrixLawQuad, \
    MatrixLawQuadratic, MatrixLawBase
from bmcs_beam.mxn.mxn_class_extension import \
    MxNClassExt
from traits.api import \
    Property, cached_property, Dict, Str, \
    Float
from traitsui.api import \
    View, Item, EnumEditor

from .material_type_base import \
    MaterialTypeBase
from .material_type_handler import \
    MaterialTypeHandler


class MTMatrixMixture(MaterialTypeBase):
    '''Base class for concrete constitutive laws.'''

    f_ck = Float(55.7, auto_set=False, enter_set=True,
                 law_input=True)
    '''Ultimate compression stress  [MPa]
    '''

    eps_c_u = Float(0.0033, auto_set=False, enter_set=True,
                    law_input=True)
    '''Strain at failure of the matrix in compression [-]
    '''

    E_c = Float(28e+3, auto_set=False, enter_set=True,
                law_input=True)

    mtrl_laws = Dict((Str, MatrixLawBase))

    def _mtrl_laws_default(self):
        basic_laws = {'bilinear':
                      MatrixLawBilinear(f_ck=55.7, eps_c_u=0.0033),
                      'constant':
                      MatrixLawBlock(f_ck=55.7, eps_c_u=0.0033,
                                     high_strength_level=50.0, E_c=28e+3),
                      'linear':
                      MatrixLawLinear(),
                      'quad':
                      MatrixLawQuad(f_ck=55.7, eps_c_u=0.0033,
                                    high_strength_level=50.0, E_c=28e+3),
                      'quadratic':
                      MatrixLawQuadratic(f_ck=55.7, eps_c_u=0.0033,
                                         high_strength_level=50.0, E_c=28e+3)
                      }
        return basic_laws

    possible_laws = {'bilinear': MatrixLawBilinear, 'constant': MatrixLawBlock,
                     'linear': MatrixLawLinear, 'quad': MatrixLawQuad,
                     'quadratic': MatrixLawQuadratic}

    named_mtrl_laws = Property(depends_on='mtrl_laws,+law_input')

    @cached_property
    def _get_named_mtrl_laws(self):
        for key, mtrl_law in list(self.mtrl_laws.items()):
            mtrl_law.node_name = key
            mtrl_law.f_ck = self.f_ck
            mtrl_law.eps_c_u = self.eps_c_u
            mtrl_law.E_c = self.E_c
        return self.mtrl_laws

    #=========================================================================
    # UI-related functionality
    #=========================================================================

    tree_view = View(Item('f_ck'),
                     Item('eps_c_u'),
                     Item('E_c'),
                     Item('new_law', show_label=False),
                     Item('chosen_law',
                          editor=EnumEditor(name='law_keys'),
                          show_label=False),
                     Item('del_law', show_label=False),
                     handler=MaterialTypeHandler()
                     )

MTMatrixMixture.db = MxNClassExt(
    klass=MTMatrixMixture,
    verbose='io',
    node_name='Matrix mixtures'
)

if MTMatrixMixture.db.get('default_mixture', None):
    del MTMatrixMixture.db['default_mixture']
MTMatrixMixture.db['default_mixture'] = MTMatrixMixture()



'''
Created on 6. 4. 2014

@author: Vancikv

Standard tree view with the default database node
and added parametric study node
'''

from bmcs_beam.mxn.use_cases import \
    UseCaseContainer, UCParametricStudy
from bmcs_beam.mxn.view import \
    MxNTreeView


mxn_ps = UCParametricStudy()
mxn_ps.element_to_add = 'mxndiagram'
mxn_ps.add_element = True
mxn_ps.tree_node_list[-1].node_name = 'Study #1'
mxn_ps.add_element = True
mxn_ps.tree_node_list[-1].node_name = 'Study #2'

ucc = UseCaseContainer()
ucc.tree_node_list.append(mxn_ps)

mxn_ps_view = MxNTreeView(root=ucc)
mxn_ps_view.selected_node = mxn_ps
mxn_ps_view.replot = True
mxn_ps_view.configure_traits()



#-------------------------------------------------------------------------------
#
# Copyright (c) 2009, IMB, RWTH Aachen.
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in simvisage/LICENSE.txt and may be redistributed only
# under the conditions described in the aforementioned license.  The license
# is also available online at http://www.simvisage.com/licenses/BSD.txt
#
# Thanks for using Simvisage open source!
#
# Created on Jan 19, 2010 by: rch

from traits.api import \
    Array, Bool, Enum, Float, HasTraits, \
    Instance, Int, Trait, Str, Enum, \
    Callable, List, TraitDict, Any, Range, \
    DelegatesTo, PrototypedFrom, \
    Event, on_trait_change, Button, \
    Interface, implements, Property, cached_property
                                 
from traitsui.api import \
    View, Item, TabularEditor, Group
                              
from numpy import \
    linspace, arange
   
from .i_sim_model import \
    ISimModel

from etsproxy.traits.ui.tabular_adapter import \
    TabularAdapter, AnITabularAdapter

class LevelListAdapter( TabularAdapter ):

    columns = Property

    #---------------------------------------------------------------
    # EXTRACT FACTOR NAMES AS COLUMNS FOR TABLE EDITOR
    #-------------------------------------------------------------------
    def _get_columns(self):
        return [ ( 'level', 'index' ), ('value', 0) ]

    font        = 'Courier 10'
    alignment   = 'center'
    odd_bg_color     = 'lightgray'
    
    index_width = Float( 40 )
    index_text  = Property
    def _get_index_text ( self ):
        return str( self.row )
    
    val_width = Float( 100 )

level_list_editor = TabularEditor( adapter = LevelListAdapter() )

class SimFactor( HasTraits ):
    
    model = Instance( ISimModel )

    unit  = Str('-')

    regular = Bool
    
    levels    = List
    
    level_list = Property( depends_on = '+levels_modified' )
    @cached_property
    def _get_level_list(self):
        raise NotImplementedError

    levels_table = Property( depends_on = '+levels_modified' )
    @cached_property
    def _get_levels_table(self):
        return [ [level] for level in self.level_list ]

    def get_n_levels(self):
        return len( self.level_list )

class SimNumFactor( SimFactor ):
    '''
    Parameter definition derived from the Param and SimModel
    '''
    regular = True
    
    def get_level_value( self, v ):
        return v
    
    traits_view = View( Item('regular'),
                        Group(
                        Item('min_level', visible_when = 'regular' ),
                        Item('max_level', visible_when = 'regular' ),
                        Item('n_levels', visible_when = 'regular'  ),
                        columns = 2,
                        ),
                        Group(
                        Item('levels_table', editor = level_list_editor,
                             show_label = False,
                              style = 'custom' ),
                              ),
                        scrollable = True ) 

class SimIntFactor( SimNumFactor ):
    '''
    Parameter definition derived from the Param and SimModel
    '''
    min_level = Int( 0, auto_set = False, enter_set = True, levels_modified = True )
    max_level = Int( 1, auto_set = False, enter_set = True, levels_modified = True )
    n_levels  = Int( 1, auto_set = False, enter_set = True, levels_modified = True )
    
    def _get_level_list(self):
        if self.regular:
            step = ( self.max_level - self.min_level) / self.n_levels
            if int( step ) == 0:
                step = 1
            levels = arange( self.min_level, self.max_level + 1, step, dtype = int )                
            return levels
        else:
            return self.levels

class SimFloatFactor( SimNumFactor ):
    '''
    Parameter definition derived from the Param and SimModel
    '''
    min_level = Float( 0, auto_set = False, enter_set = True, levels_modified = True )
    max_level = Float( 1, auto_set = False, enter_set = True, levels_modified = True )
    n_levels  = Int( 2, auto_set = False, enter_set = True, levels_modified = True )
        
    def _get_level_list(self):
        if self.regular:
            return linspace( self.min_level, self.max_level, self.n_levels )
        else:
            return self.levels

class SimEnumFactor( SimFactor ):
    '''
    Parameter definition derived from the Param and SimModel
    '''
    regular = False
    levels = List( auto_set = False, enter_set = True, levels_modified = True )
    
    traits_view = View(
                              Item('levels_table', editor = level_list_editor,
                              show_label = False,
                              style = 'custom' ),
                        scrollable = True
                        ) 

    def get_level_value( self, v ):
        return getattr( self.model, v )
    
    def _get_level_list(self):
        return self.levels
    


'''
Created on 26. 2. 2014

@author: Vancikv
'''

from traits.api import \
    Float, Property, cached_property, Array, Int, List

from bmcs_beam.mxn.constitutive_law import CLBase

import numpy as np

from math import exp, log

from .reinf_law_base import \
    ReinfLawBase


class ReinfLawBilinear(ReinfLawBase):

    '''Effective crack bridge Law using a cubic polynomial.'''

    sig_tex_u = Float(1216., enter_set=True, auto_set=False, input=True)
    eps_u = Float(0.014, enter_set=True, auto_set=False, input=True)
    var_a = Float(0.8, enter_set=True, auto_set=False, input=True)
    eps_el_fraction = Float(0.0001, enter_set=True, auto_set=False, input=True)

    cnames = ['eps_u', 'var_a']

    u0 = List([0.014, 0.8])

    eps_arr = Property(depends_on='+input')

    @cached_property
    def _get_eps_arr(self):
        return np.hstack([0., self.eps_el_fraction * self.eps_u, self.eps_u])

    sig_arr = Property(depends_on='+input')

    @cached_property
    def _get_sig_arr(self):
        return np.hstack([0., self.var_a * self.sig_tex_u, self.sig_tex_u])

ReinfLawBase.db.constants['bilinear-default'] = ReinfLawBilinear()





from traits.api import HasTraits, Instance
from traitsui.api import View, Item
from promod.core.promod import IProMod

class ProModelService(HasTraits):

    # Set by envisage when this is offered as a service offer.
    window = Instance('etsproxy.pyface.workbench.api.WorkbenchWindow')

    promod = Instance(IProMod)

    def _promod_changed(self):
        '''Update the dependent services'''
        pass

    traits_view = View(Item('promod@', show_label = False),
                        resizable = True)

if __name__ == '__main__':
    promod_service = ProModService()
    promod_service.configure_traits()



'''
Created on 14. 4. 2014

@author: Vancikv
'''

from traitsui.api import \
    View, Item, HGroup, Handler, \
    UIInfo, spring

from traitsui.menu import \
    Action

from traitsui.file_dialog import \
    open_file, save_file

from traits.api import \
    Button, Instance, WeakRef

from bmcs_beam.mxn.utils import \
    get_outfile

import pickle

plot_self = Action(name='Plot', action='plot_node')
'''Context menu action for plotting tree nodes
'''
new_material = Action(name='New material', action='new_material')
'''Context menu action for adding a new database member
'''
del_material = Action(name='Delete', action='del_material')
'''Context menu action for deleting a database member
'''
menu_save = Action(name='Save', action='menu_save')
'''Menubar action for saving the root node to file
'''
menu_open = Action(name='Open', action='menu_open')
'''Menubar action for loading root node from file
'''
menu_exit = Action(name='Exit', action='menu_exit')
'''Menubar action for terminating the view
'''


class MxNTreeViewHandler(Handler):

    '''Handler for MxNTreeView class
    '''
    # The UIInfo object associated with the view:
    info = Instance(UIInfo)
    node = WeakRef

    ok = Button('OK')
    cancel = Button('Cancel')
    delete = Button('OK')
    exit_dialog = ('Do you really wish to end '
                   'the session? Any unsaved data '
                   'will be lost.')

    exit_view = View(Item(name='', label=exit_dialog),
                     HGroup(Item('ok', show_label=False, springy=True),
                            Item('cancel', show_label=False, springy=True)
                            ),
                     title='Exit dialog',
                     kind='live'
                     )

    del_view = View(
        HGroup(
            spring,
            Item(name='', label='Really delete?'),
            Item('delete', show_label=False),
            Item('cancel', show_label=False),
        ),
        kind='popup'
    )

    def new_material(self, info, node):
        mat_name = 'new_material_1'
        count = 1
        while node.get(mat_name, None):
            count += 1
            mat_name = 'new_material_' + str(count)
        node[mat_name] = node.klass()

    def del_material(self, info, node):
        if info.initialized:
            self.node = node
            self._ui = self.edit_traits(view='del_view')

    def plot_node(self, info, node):
        '''Handles context menu action Plot for tree nodes
        '''
        info.object.figure.clear()
        node.plot(info.object.figure)
        info.object.data_changed = True

    def menu_save(self, info):
        file_name = get_outfile(folder_name='.mxn', file_name='')
        file_ = save_file(file_name=file_name)
        if file_:
            pickle.dump(info.object.root, open(file_, 'wb'), 1)

    def menu_open(self, info):
        file_name = get_outfile(folder_name='.mxn', file_name='')
        file_ = open_file(file_name=file_name)
        if file_:
            info.object.root = pickle.load(open(file_, 'rb'))

    def menu_exit(self, info):
        if info.initialized:
            self.info = info
            self._ui = self.edit_traits(view='exit_view')

    def _delete_fired(self):
        del self.node.db[self.node.key]
        self._ui.dispose()

    def _ok_fired(self):
        self._ui.dispose()
        self.info.ui.dispose()

    def _cancel_fired(self):
        self._ui.dispose()



'''
Created on 26. 2. 2014

@author: Vancikv
'''

from traits.api import \
    Property, cached_property

import numpy as np

from bmcs_beam.mxn.mfn import MFnLineArray

import sympy as sp

from .matrix_law_base import \
    MatrixLawBase

a_, b_, c_, x_ = sp.symbols('a,b,c,x')


class MatrixLawQuad(MatrixLawBase):

    '''Effective crack bridge Law using a cubic polynomial.'''

    mfn = Property(depends_on='+input')

    @cached_property
    def _get_mfn(self):
        '''quadratic stress-strain-diagram of the concrete
        '''

        quad_fn = a_ * x_ ** 2 + b_ * x_ + c_

        eq1 = quad_fn.subs(x_, 0)
        eq2 = quad_fn.subs(x_, self.eps_c_u) - self.f_ck
        eq3 = sp.diff(quad_fn, x_).subs(x_, 0) - self.E_c
        sol = sp.solve([eq1, eq2, eq3], a_, b_, c_)
        sig_eps_fn = sp.lambdify([x_], quad_fn.subs(sol))
        eps_arr = np.linspace(0., self.eps_c_u, num=100.)
        xdata = eps_arr
        ydata = sig_eps_fn(eps_arr)

        return MFnLineArray(xdata=xdata, ydata=ydata)



#-------------------------------------------------------------------------------
#
# Copyright (c) 2009, IMB, RWTH Aachen.
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in simvisage/LICENSE.txt and may be redistributed only
# under the conditions described in the aforementioned license.  The license
# is also available online at http://www.simvisage.com/licenses/BSD.txt
#
# Thanks for using Simvisage open source!
#
# Created on Jan 19, 2010 by: rch

from traits.api import Array, Bool, Enum, Float, HasTraits, \
                                 Instance, Int, Trait, Str, Enum, \
                                 Callable, List, TraitDict, Any, Range, \
                                 Delegate, Event, on_trait_change, Button, \
                                 Interface, implements, Property, cached_property
                                 
class ISimArray( Interface ):
    '''
    Interface of the boundary condition.
    '''




from math import sin, cos, sqrt

from numpy import linspace, frompyfunc
from traits.api import \
    Array, Bool, Callable, Enum, Float, HasTraits, \
    Instance, Int, Trait, Str, Enum, Callable, List, TraitDict, Any, \
    Dict, Property, cached_property, WeakRef, Delegate, \
    ToolbarButton, on_trait_change, Code, Expression, Button
from traitsui.api import \
    Item, View, HGroup, ListEditor, VGroup, VSplit, Group, HSplit
from traitsui.menu import \
    NoButtons, OKButton, CancelButton, Action, CloseAction, Menu, \
    MenuBar, Separator

from bmcs_beam.mxn.mfn import MFnLineArray
from bmcs_beam.mxn.mfn.mfn_line.mfn_chaco_editor import MFnChacoEditor
from bmcs_beam.mxn.mfn.mfn_line.mfn_matplotlib_editor import MFnMatplotlibEditor
from bmcs_beam.mxn.mfn.mfn_line.mfn_plot_adapter import MFnPlotAdapter


a = MFnPlotAdapter(max_size=(300, 200),
                   padding={'top': 0.95,
                            'left': 0.1,
                            'bottom': 0.1,
                            'right': 0.95},
                   line_color=['orange'])


class AnalyticalFunction(HasTraits):

    expression = Expression('x**2', auto_set=False, enter_set=True)
    refresh = Button('redraw')

    def _refresh_fired(self):
        xdata = linspace(0.001, 10, 10000)
        fneval = frompyfunc(lambda x: eval(self.expression), 1, 1)
        ydata = fneval(xdata)
        self.mfn.set(xdata=xdata, ydata=ydata)
        self.mfn.data_changed = True

    mfn = Instance(MFnLineArray)

    def _mfn_default(self):
        return MFnLineArray()

    @on_trait_change('expression')
    def update_mfn(self):
        self._refresh_fired()

    view_mpl = View(HGroup(Item('expression'), Item('refresh')),
                    Item('mfn', editor=MFnMatplotlibEditor(adapter=a),
                         show_label=False),
                    resizable=True,
                    scrollable=True,
                    height=0.5, width=0.5
                    )

    view_chaco = View(HGroup(Item('expression'), Item('refresh')),
                      Item('mfn', editor=MFnChacoEditor(adapter=a),
                           resizable=True, show_label=False),
                      resizable=True,
                      scrollable=True,
                      height=0.3, width=0.3
                      )

if __name__ == '__main__':
    fn = AnalyticalFunction()
    fn._refresh_fired()
    fn.configure_traits(view="view_mpl", kind='nonmodal')
  #  fn.configure_traits( view = "view_chaco", kind = 'nonmodal' )



'''
Created on 14. 4. 2014

@author: Vancikv
'''

from traits.api import \
    HasStrictTraits, Str, List, WeakRef, \
    Property, cached_property

from traitsui.api import \
    View

class MxNLeafNode(HasStrictTraits):
    '''Base class of all model classes that can appear in a tree view.
    '''
    node_name = Str('<unnamed>')

    def plot(self, fig):
        return

class MxNTreeNode(HasStrictTraits):
    '''Base class of all model classes that can appear in a tree view.
    '''
    node_name = Str('<unnamed>')

    tree_node_list = List([])

    tree_view = View()

    def append_node(self, node):
        '''Add a new subnode to the current node.
        Inform the tree view to select the new node within the view.
        '''
        self.tree_node_list.append(node)

    def plot(self, fig):
        '''Plot the content of the current node.
        '''
        return

class ReinfLayoutTreeNode(MxNTreeNode):
    '''Class accommodating the list of all reinforcement components.
    '''
    node_name = Str('Reinforcement layout')

    cs_state = WeakRef(HasStrictTraits)

    def __getstate__ (self):
        '''Overriding __getstate__ because of WeakRef usage
        '''
        state = super(HasStrictTraits, self).__getstate__()

        for key in [ 'cs_state', 'cs_state_' ]:
            if key in state:
                del state[ key ]

        return state


    def plot(self, fig):
        ax = fig.add_subplot(1, 1, 1)
        self.cs_state.plot_geometry(ax)

    tree_node_list = Property(depends_on='cs_state.reinf_components_with_state')
    @cached_property
    def _get_tree_node_list(self):
        self.cs_state.changed = True
        return self.cs_state.reinf_components_with_state



'''
Created on 25. 2. 2014

@author: Vancikv
'''

from traits.api import \
    Float, Property, cached_property

from traitsui.api import \
    View, Item, Group

from .matrix_cross_section_geo import MCSGeo

import numpy as np


class MCSGeoCirc(MCSGeo):

    '''Circular shaped cross section
    '''

    radius = Float(0.3, auto_set=False, enter_set=True, geo_input=True)
    '''radius of cross section
    '''

    height = Property(depends_on='radius')
    '''Height of cross section
    '''
    @cached_property
    def _get_height(self):
        return 2 * self.radius

    width = Property(depends_on='radius')
    '''Width of cross section
    '''
    @cached_property
    def _get_width(self):
        return 2 * self.radius

    gravity_centre = Property(depends_on='+geo_input')
    '''z distance of gravity centre from upper rim
    '''
    @cached_property
    def _get_gravity_centre(self):
        return self.radius

    def get_width(self, z):
        '''returns width of cross section for given vertical coordinate
        '''
        r_dist = z - self.radius
        '''transfer distance from top to distance from center
        '''
        width = 2 * np.sqrt(self.radius ** 2 - r_dist ** 2)
        return width

    width_vct = Property()

    @cached_property
    def _get_width_vct(self):
        return np.vectorize(self.get_width, otypes=[np.float])

    def plot_geometry(self, ax):
        '''Plot geometry'''
        dx, dz = self.radius, self.radius
        fi_outline_arr = np.linspace(0, 2 * np.pi, 60)

        ax.plot(np.cos(fi_outline_arr) * self.radius + dx,
                np.sin(fi_outline_arr) * self.radius + dz, color='blue')
        ax.axis('equal')
        ax.axis([dx - self.radius * 1.1, dx + self.radius * 1.1,
                 dz - self.radius * 1.1, dz + self.radius * 1.1])

    view = View(Item('radius'),
                resizable=True,
                buttons=['OK', 'Cancel'])



'''
Created on 28. 3. 2014

The module defines the framework for tree visualization of the model
in a ModelView window. The components of the model classes should
inherit from the MxNTreeNode and supply the attributes

 - node_name - a label to appear in the tree editor
 - tree_node_list  - subnodes of the current nodes

 Further, the plot method can be defined to plot the current node
 in the plot window of the model view.

@author: rch
'''

from matplotlib.figure import \
    Figure
from bmcs_beam.mxn.mxn_tree_node import \
    MxNTreeNode, MxNLeafNode
from traits.api import \
    HasStrictTraits, Instance, Button, Event
from traits.etsconfig.api import ETSConfig
from traitsui.api import \
    TreeEditor, TreeNode, View, Item, Group, \
    HSplit, HGroup
from traitsui.menu import \
    Menu, MenuBar, Separator

from bmcs_beam.mxn.utils.extra_traits.editors.mpl_figure_editor import \
    MPLFigureEditor
from .mxn_tree_customized_nodes import \
    custom_node_list
from .mxn_tree_view_handler import \
    MxNTreeViewHandler, plot_self, menu_save, \
    menu_open, menu_exit


if ETSConfig.toolkit == 'wx':
    from traitsui.wx.tree_editor import \
        DeleteAction
if ETSConfig.toolkit == 'qt4':
    from traitsui.qt4.tree_editor import \
        DeleteAction
else:
    raise ImportError("tree actions for %s toolkit not availabe" % \
        ETSConfig.toolkit)

tree_node = TreeNode(node_for=[MxNTreeNode],
                     auto_open=False,
                     children='tree_node_list',
                     label='node_name',
                     view='tree_view',
                     menu=Menu(plot_self, DeleteAction),
                     )

leaf_node = TreeNode(node_for=[MxNLeafNode],
                     auto_open=True,
                     children='',
                     label='node_name',
                     view='tree_view',
                     menu=Menu(plot_self)
                     )

tree_editor = TreeEditor(
    nodes=custom_node_list + [tree_node, leaf_node],
    selected='selected_node',
    orientation='vertical'
)


class MxNTreeView(HasStrictTraits):

    '''View object for a cross section state.
    '''
    root = Instance(MxNTreeNode)

    selected_node = Instance(HasStrictTraits)

    figure = Instance(Figure)

    def _figure_default(self):
        figure = Figure(facecolor='white')
        figure.add_axes([0.08, 0.13, 0.85, 0.74])
        return figure

    data_changed = Event

    replot = Button

    def _replot_fired(self):
        self.figure.clear()
        self.selected_node.plot(self.figure)
        self.data_changed = True

    clear = Button()

    def _clear_fired(self):
        self.figure.clear()
        self.data_changed = True

    view = View(HSplit(Group(Item('root',
                                  editor=tree_editor,
                                  resizable=True,
                                  show_label=False,
                                  width=400,
                                  height=400),
                             ),
                       Group(HGroup(Item('replot', show_label=False),
                                    Item('clear', show_label=False),
                                    ),
                             Item('figure', editor=MPLFigureEditor(),
                                  resizable=True, show_label=False,
                                  springy=True),
                             label='plot sheet',
                             dock='tab',
                             )
                       ),
                id='mxntreeview_id',
                width=0.9,
                height=0.8,
                title='MxN',
                resizable=True,
                handler=MxNTreeViewHandler(),
                menubar=MenuBar(Menu(menu_exit, Separator(),
                                     menu_save, menu_open,
                                     name='File'))
                )

if __name__ == '__main__':

    tr = MxNTreeNode(node_name='root',
                     tree_node_list=[MxNTreeNode(node_name='subnode 1'),
                                     MxNTreeNode(node_name='subnode 2'),
                                     ])

    tv = MxNTreeView(root=tr)
    tv.configure_traits()



'''
Created on 2. 2. 2014

@author: Vancikv

Example of cross section assembly - Circular concrete
cross section with combined reinforcement of textile
layers and steel bars
'''

from bmcs_beam.mxn.cross_section import \
    CrossSection

from bmcs_beam.mxn.matrix_cross_section import \
    MatrixCrossSection, MCSGeoCirc

from bmcs_beam.mxn.reinf_layout import \
    RLCTexLayer, RLCBar

from matplotlib.figure import \
    Figure

from matplotlib.backends.backend_agg import \
    FigureCanvasAgg

from bmcs_beam.mxn.utils import \
    get_outfile

import numpy as np

radius = 0.3
ge = MCSGeoCirc(radius=0.3)
'''Cross section geometry
'''

n_bars = 10
radius_bars = 0.26
'''Specifications of bar reinforcement
'''

fi_bar_arr = np.arange(0, 2 * np.pi, 2 * np.pi / n_bars,
                       dtype=float)
x_bar_arr = np.cos(fi_bar_arr) * radius_bars + radius
z_bar_arr = radius - np.sin(fi_bar_arr) * radius_bars
'''Positions of bars divided equally across the
circumference
'''

bar_lst = [RLCBar(x=x_bar_arr[i],
                  z=z_bar_arr[i],
                  material='bar_d10',
                  material_law='steel'
                  ) for i in range(n_bars)]
'''List of bars
'''

tl1 = RLCTexLayer(z_coord=0.15, material='default_fabric',
                  material_law='fbm')
tl2 = RLCTexLayer(z_coord=0.16, material='default_fabric',
                  material_law='fbm')
'''Two layers of textile reinforcement
'''

mcs = MatrixCrossSection(geo=ge,
                         n_cj=20,
                         material='default_mixture',
                         material_law='constant')
'''Matrix cross section
'''

cs = CrossSection(reinf=[tl1, tl2] + bar_lst,
                  matrix_cs=mcs,
                  eps_lo=0.002,
                  eps_up=-0.0033,
                  )

print('normal force', cs.N)
print('moment', cs.M)

fig = Figure(figsize=(10, 7), dpi=80, facecolor='white')
canvas = FigureCanvasAgg(fig)
ax = fig.add_subplot(1, 1, 1)
cs.plot_geometry(ax)

canvas.print_figure(get_outfile(folder_name='.mxn',
                                file_name='ex04.png'))



'''
Created on Sep 4, 2012

@todo: introduce the dock feature for the views
@todo: classify the state changes and provide examples.

@author: rch
'''
from traits.api import \
    HasStrictTraits, Property, \
    Event, on_trait_change, WeakRef, \
    Constant, cached_property, Bool

from bmcs_beam.mxn.cross_section_state import \
    CrossSectionState

from bmcs_beam.mxn.mxn_tree_node import \
    MxNTreeNode

from .utils import \
    KeyRef

COMPONENT_CHANGE = '+geo_input,geo.changed,material_changed,law_changed,material,material_law'


class CrossSectionComponent(MxNTreeNode):

    '''Cross section component supplying the normal force and moment..
    '''

    def __init__(self, *args, **metadata):
        default_material = metadata.get('material', None)
        if default_material:
            self.material = default_material
        else:
            print('Warning: material not specified for object %s' % self)
        '''It is necessary to set default value of material here to ensure
        it has been assigned when its editor is requested by the UI
        '''

        self.add_trait(
            'material_law', KeyRef(db=self.material_.named_mtrl_laws))
        self.on_trait_change(
            self._refresh_material_law, 'material,material_changed')
        '''The scope of material laws is dependent on the chosen material
        '''

        default_material_law = metadata.get(
            'material_law', list(self.material_.named_mtrl_laws.keys())[0])
        self.material_law = default_material_law

        super(CrossSectionComponent, self).__init__(**metadata)

    def _refresh_material_law(self):
        '''material_law is redefined upon change of material
        '''
        named_laws = self.material_.named_mtrl_laws
        val = self.material_law
        self.add_trait('material_law', KeyRef(db=named_laws))
        del self.material_law
        try:
            self.material_law = val
        except:
            self.material_law = list(self.material_.named_mtrl_laws.keys())[0]

    state = WeakRef(CrossSectionState, transient=True)
    '''Strain state of a cross section
    '''

    def __getstate__(self):
        '''Overriding __getstate__ because of WeakRef usage
        '''
        state = super(CrossSectionComponent, self).__getstate__()

        for key in ['state', 'state_']:
            if key in state:
                del state[key]

        return state

    def __setstate__(self, state):
        '''Overriding __setstate__ to ensure dynamic attributes exist.
        '''
        self.__init__(**state)

    unit_conversion_factor = Constant(1000.0)

    material_changed = Event
    '''Notifier set to True when internal values of material change
    '''

    law_changed = Event
    '''Notifier set to True when internal values of constitutive law change
    '''
    eps_changed = Event
    '''State notifier that is set to true if the cross section state has changed
    upon modifications of eps_lo and eps_up
    '''

    @on_trait_change(COMPONENT_CHANGE)
    def notify_change(self):
        '''Propagate the change of the component geometry or stress-strain
        law to the cross section state.
        '''
        if self.state:
            self.state.changed = True

    #=========================================================================
    # Cross-sectional stress resultants
    #=========================================================================

    N = Property()
    '''Resulting normal force.
    '''

    M = Property()
    '''Resulting moment.
    '''

    #=========================================================================
    # Auxiliary methods for tree editor
    #=========================================================================
    tree_node_list = Property(depends_on='material,material_law')

    @cached_property
    def _get_tree_node_list(self):
        return [self.material_law_]

if __name__ == '__main__':
    pass



'''
Created on 23. 4. 2014

@author: Vancikv
'''

from bmcs_beam.mxn.mxn_class_extension import \
    MxNClassExt
from bmcs_beam.mxn.reinf_laws import \
    ReinfLawBase, ReinfLawSteel
from traits.api import \
    Property, cached_property, Dict, Str, \
    Float
from traitsui.api import \
    View, Item, EnumEditor

from .material_type_base import \
    MaterialTypeBase
from .material_type_handler import \
    MaterialTypeHandler


class MTReinfBar(MaterialTypeBase):

    area = Float(0.0000785, auto_set=False, enter_set=True, geo_input=True)
    '''Cross section area [m**2]'''

    mtrl_laws = Dict((Str, ReinfLawBase))

    def _mtrl_laws_default(self):
        basic_laws = {'steel':
                      ReinfLawSteel(f_yk=500., E_s=200000., eps_u=0.025),
                      }
        return basic_laws

    possible_laws = {'steel': ReinfLawSteel}

    named_mtrl_laws = Property(depends_on='mtrl_laws')

    @cached_property
    def _get_named_mtrl_laws(self):
        for key, mtrl_law in list(self.mtrl_laws.items()):
            mtrl_law.node_name = key
        return self.mtrl_laws

    #=========================================================================
    # UI-related functionality
    #=========================================================================

    tree_view = View(Item('area'),
                     Item('new_law', show_label=False),
                     Item('chosen_law',
                          editor=EnumEditor(name='law_keys'),
                          show_label=False),
                     Item('del_law', show_label=False),
                     handler=MaterialTypeHandler()
                     )

MTReinfBar.db = MxNClassExt(
    klass=MTReinfBar,
    verbose='io',
    node_name='Reinforcement bars'
)

if MTReinfBar.db.get('bar_d10', None):
    del MTReinfBar.db['bar_d10']
MTReinfBar.db['bar_d10'] = MTReinfBar(area=0.0000785)



'''
Created on Nov 21, 2013

@author: rch
'''

from bmcs_beam.mxn.cross_section import \
    CrossSection

from bmcs_beam.mxn.ecb_calib import \
    ECBCalib

from bmcs_beam.mxn.matrix_cross_section import \
    MatrixCrossSection, MCSGeoRect

from bmcs_beam.mxn.reinf_layout import \
    RLCTexUniform

import numpy as np

import pickle

from bmcs_beam.mxn.utils import \
    get_outfile

def test_ecb_law_calib():
    '''Test the calibrated crack bridge law.
    '''
    #===============================================================================
    # Input values
    #===============================================================================
    # 23 rovings
    n_rovings = 23
    # roving cross sectional area
    A_rov = 0.461
    # ultimate moment
    Mu = 3.50

    ge = MCSGeoRect(height=0.06, width=0.2)
    mcs = MatrixCrossSection(geo=ge, n_cj=20, material='default_mixture', material_law='constant')

    uni_layers = RLCTexUniform(n_layers=12, material='default_fabric', material_law='fbm')

    uni_layers.material_.set(s_0=0.0083, A_roving=0.461)

    cs = CrossSection(matrix_cs=mcs, reinf=[uni_layers])

    calib = ECBCalib(cs=cs, Mu=Mu)
    test_eps_arr = np.linspace(0.0, 0.005, 10)
    calib_file = get_outfile(folder_name='.mxn',
                             file_name='test06_calib.pkl')

    assert np.allclose(calib.calibrated_ecb_law.mfn_vct(test_eps_arr),
                       np.array([0., 169.56745719, 308.0713995, 426.15150252,
                                 528.22368093, 617.57902395, 696.18106351,
                                 765.44225452, 826.76851637, 881.07603382],
                                dtype=float))

    assert np.allclose(cs.reinf_components_with_state[0].layer_lst[0].material_law_.mfn_vct(test_eps_arr),
                       np.array([0., 169.56745719, 308.0713995, 426.15150252,
                                 528.22368093, 617.57902395, 696.18106351,
                                 765.44225452, 826.76851637, 881.07603382],
                                dtype=float))
    pickle.dump(calib, open(calib_file, 'wb'), 1)

    uni_layers.material_law = 'cubic'

    assert np.allclose(calib.calibrated_ecb_law.mfn_vct(test_eps_arr),
                       np.array([0., 128.33734405, 247.41169038, 357.44885771,
                                 458.78005031, 551.89774542, 636.94074038,
                                 714.45215726, 784.72942118, 848.06386907],
                                dtype=float))

    uni_layers.material_law = 'linear'

    assert np.allclose(calib.calibrated_ecb_law.mfn_vct(test_eps_arr),
                       np.array([0., 51.40222925, 102.8044585, 154.20668775,
                                 205.608917, 257.01114625, 308.41337551,
                                 359.81560476, 411.21783401, 462.62006326],
                                dtype=float))

    uni_layers.material_law = 'bilinear'

    assert np.allclose(calib.calibrated_ecb_law.mfn_vct(test_eps_arr),
                       np.array([0., 858.01243898, 873.058472, 888.10450501,
                                 903.15053803, 918.19657105, 933.24260407,
                                 948.28863708, 963.3346701, 978.38070312],
                                dtype=float))

    mcs.material_law = 'quadratic'
    uni_layers.material_law = 'fbm'
    calib.Mu = 3.49

    assert np.allclose(calib.calibrated_ecb_law.mfn_vct(test_eps_arr),
                       np.array([0., 300.73451126, 489.25642137, 628.66800959,
                                 737.65269075, 825.20174306, 896.67226786,
                                 955.78175683, 1004.84170587, 1045.86868703],
                                dtype=float))

    loaded_calib = pickle.load(open(calib_file, 'rb'))

    assert np.allclose(loaded_calib.calibrated_ecb_law.mfn_vct(test_eps_arr),
                       np.array([0., 169.56745719, 308.0713995, 426.15150252,
                                 528.22368093, 617.57902395, 696.18106351,
                                 765.44225452, 826.76851637, 881.07603382],
                                dtype=float))

if __name__ == '__main__':
    test_ecb_law_calib()



#-------------------------------------------------------------------------
#
# Copyright (c) 2009, IMB, RWTH Aachen.
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in simvisage/LICENSE.txt and may be redistributed only
# under the conditions described in the aforementioned license.  The license
# is also available online at http://www.simvisage.com/licenses/BSD.txt
#
# Thanks for using Simvisage open source!
#
# Created on Feb 15, 2010 by: rch

import os
from os.path import expanduser
from traits.api import \
    HasTraits, Directory, List, Int, Float, Any, \
    on_trait_change, File, Constant, Instance, Trait, \
    Array, Str, Property, cached_property, WeakRef, \
    Dict, Button, Bool, Enum, Event
from traitsui.api import \
    View, Item, DirectoryEditor, TabularEditor, HSplit, VGroup, \
    TableEditor, EnumEditor, Handler, FileEditor, VSplit, Group
from traitsui.menu import \
    OKButton, CancelButton
from traitsui.table_column import \
    ObjectColumn

from .ex_run import ExRun


class ExTreatment(HasTraits):

    runs = List()

    #--------------------------------------------------------------------
    # file containing the association between the factor combinations
    # and data files having the data
    #--------------------------------------------------------------------
    data_file = File

    def _data_file_default(self):
        home_dir = os.environ['HOME']
        dat_dir = os.path.join(
            home_dir, 'workspace', 'simvisage', 'src', 'apps')
        return dat_dir



import numpy as np
from traits.api import Array, Float, Event, HasTraits, \
                                 ToolbarButton, on_trait_change, \
                                 Property, cached_property, Enum

from scipy import interpolate as ip

class MFnLineArray(HasTraits):

    # Public Traits
    xdata = Array(float, value = [0.0, 1.0])
    def _xdata_default(self):
        '''
        convenience default - when xdata not defined created automatically as
        an array of integers with the same shape as ydata
        '''
        return np.arange(self.ydata.shape[0])

    ydata = Array(float, value = [0.0, 1.0])

    extrapolate = Enum('constant', 'exception', 'diff', 'zero')

    # alternative vectorized interpolation using scipy.interpolate   
    def get_values(self, x, k = 1):
        '''
        vectorized interpolation, k is the spline order, default set to 1 (linear)
        '''
        tck = ip.splrep(self.xdata, self.ydata, s = 0, k = k)

        x = np.array([x]).flatten()

        if self.extrapolate == 'diff':
            values = ip.splev(x, tck, der = 0)
        elif self.extrapolate == 'exception':
            if x.all() < self.xdata[0] and x.all() > self.xdata[-1]:
                values = values = ip.splev(x, tck, der = 0)
            else:
                raise ValueError('value(s) outside interpolation range')

        elif self.extrapolate == 'constant':
            values = ip.splev(x, tck, der = 0)
            values[x < self.xdata[0]] = self.ydata[0]
            values[x > self.xdata[-1]] = self.ydata[-1]
        elif self.extrapolate == 'zero':
            values = ip.splev(x, tck, der = 0)
            values[x < self.xdata[0]] = 0.0
            values[x > self.xdata[-1]] = 0.0
        return values

    def get_value(self, x):
        x2idx = self.xdata.searchsorted(x)
        if x2idx == len(self.xdata):
            x2idx -= 1
        x1idx = x2idx - 1
        x1 = self.xdata[ x1idx ]
        x2 = self.xdata[ x2idx ]
        dx = x2 - x1
        y1 = self.ydata[ x1idx ]
        y2 = self.ydata[ x2idx ]
        dy = y2 - y1
        y = y1 + dy / dx * (x - x1)
        return y

    data_changed = Event

    def get_diffs(self, x, k = 1, der = 1):
        '''
        vectorized interpolation, der is the nth derivative, default set to 1;
        k is the spline order of the data inetrpolation, default set to 1 (linear)
        '''
        xdata = np.sort(np.hstack((self.xdata, x)))
        idx = np.argwhere(np.diff(xdata) == 0).flatten()
        xdata = np.delete(xdata, idx)
        tck = ip.splrep(xdata, self.get_values(xdata, k = k), s = 0, k = k)
        return ip.splev(x, tck, der = der)

    def get_diff(self, x):
        x2idx = self.xdata.searchsorted(x)
        if x2idx == len(self.xdata):
            x2idx -= 1
        x1idx = x2idx - 1
        x1 = self.xdata[ x1idx ]
        x2 = self.xdata[ x2idx ]
        dx = x2 - x1
        y1 = self.ydata[ x1idx ]
        y2 = self.ydata[ x2idx ]
        dy = y2 - y1
        return dy / dx

    dump_button = ToolbarButton('Print data',
                                style = 'toolbar')
    @on_trait_change('dump_button')
    def print_data(self, event = None):
        print('x = ', repr(self.xdata))
        print('y = ', repr(self.ydata))

    integ_value = Property(Float(), depends_on = 'ydata')
    @cached_property
    def _get_integ_value(self):
        _xdata = self.xdata
        _ydata = self.ydata
        # integral under the stress strain curve
        return np.trapz(_ydata, _xdata)

    def clear(self):
        self.xdata = np.array([])
        self.ydata = np.array([])

    def plot(self, axes, *args, **kw):
        self.mpl_plot(axes, *args, **kw)

    def mpl_plot(self, axes, *args, **kw):
        '''plot within matplotlib window'''
        axes.plot(self.xdata, self.ydata, *args, **kw)

if __name__ == '__main__':
    import pylab as plt

#    from matplotlib import pyplot as plt
    x = np.linspace(-2, 7, 20)
    xx = np.linspace(-4, 8, 100)
    y = np.sin(x)

    mf = MFnLineArray(xdata = x, ydata = y)

    # plots raw data
    def data():
        plt.plot(x, y, 'ro', label = 'data')

    # plots interpolation and extrapolation using scalar methods
    def scalar():
        plt.plot(xx, [mf.get_value(xi) for xi in xx], label = 'values scalar')
        plt.plot(xx, [mf.get_diff(xi) for xi in xx], label = 'diff scalar')

    # plots values with extrapolation as constant value
    def constant():
        mf.extrapolate = 'constant'
        plt.plot(xx, mf.get_values(xx), label = 'constant')
        plt.plot(xx, mf.get_diffs(xx,), label = 'constant diff')

    # plots values with extrapolation as zero
    def zero():
        mf.extrapolate = 'zero'
        plt.plot(xx, mf.get_values(xx), label = 'zero')
        plt.plot(xx, mf.get_diffs(xx,), label = 'zero diff')

    # plots values with extrapolation with constant slope
    def diff():
        mf.extrapolate = 'diff'
        plt.plot(xx, mf.get_values(xx), label = 'diff')
        plt.plot(xx, mf.get_diffs(xx,), label = 'diff diff')

    # raises an exception if data are outside the interpolation range
    def exception():
        mf.extrapolate = 'exception'
        plt.plot(xx, mf.get_values(xx), label = 'diff')

    data()
    #scalar()
    #constant()
    zero()
    #diff()
    #exception()
    plt.legend(loc = 'best')
    plt.show()



#!/usr/bin/env python
'''
Created on Dec 17, 2016

Toplevel script to start oricreate 

@author: rch
'''

import sys


# A oricreate mayavi instance so we can close it correctly.
oricreate = None


def main():
    """This starts up the oricreate application.
    """
    global mxn

    # Make sure '.' is in sys.path
    if '' not in sys.path:
        sys.path.insert(0, '')
    # Start the app.
    from traits.etsconfig.api import ETSConfig
    # Check that we have a traits backend installed
    from traitsui.toolkit import toolkit
    toolkit()  # This forces the selection of a toolkit.
    if ETSConfig.toolkit in ('null', ''):
        raise ImportError('''Could not import backend for traits
________________________________________________________________________________
Make sure that you have either the TraitsBackendWx or the TraitsBackendQt
projects installed. If you installed Oricreate with easy_install, try easy_install
<pkg_name>. easy_install Oricreate[app] will also work.
If you performed a source checkout, be sure to run 'python setup.py install'
in Traits, TraitsGUI, and the Traits backend of your choice.
Also make sure that either wxPython or PyQT is installed.
wxPython: http://www.wxpython.org/
PyQT: http://www.riverbankcomputing.co.uk/software/pyqt/intro
'''
                          )

    from bmcs_beam.mxn.view import \
        MxNTreeView

    from bmcs_beam.mxn.use_cases import \
        UseCaseContainer, UCParametricStudy

    mxn_ps = UCParametricStudy()
    mxn_ps.element_to_add = 'mxndiagram'
    mxn_ps.add_element = True
    mxn_ps.tree_node_list[-1].content.cs.matrix_cs.geo.height = 0.06
    mxn_ps.tree_node_list[-1].node_name = 'Study #1 - height 6 cm'
    mxn_ps.add_element = True
    mxn_ps.tree_node_list[-1].linestyle = 'dashed'
    mxn_ps.tree_node_list[-1].content.cs.matrix_cs.geo.height = 0.07
    mxn_ps.tree_node_list[-1].node_name = 'Study #2 - height 7 cm'

    ucc = UseCaseContainer()
    ucc.tree_node_list.append(mxn_ps)

    mxn_ps_view = MxNTreeView(root=ucc)
    mxn_ps_view.selected_node = mxn_ps
    mxn_ps_view.replot = True
    mxn_ps_view.configure_traits()


def close():
    """ This closes the oricreate application.
    """
    oricreate.window.close()

if __name__ == '__main__':
    main()



'''
Created on Sep 4, 2012

@author: rch
'''

from traits.api import \
    WeakRef

from bmcs_beam.mxn.matrix_cross_section import \
    MatrixCrossSection

from bmcs_beam.mxn.cross_section_component import \
    CrossSectionComponent

STATE_AND_GEOMETRY_CHANGE = 'eps_changed,+geo_input,matrix_cs.geo.changed'
STATE_LAW_AND_GEOMETRY_CHANGE = 'eps_changed,+geo_input,matrix_cs.geo.changed,material_changed,law_changed,material,material_law'

class ReinfLayoutComponent(CrossSectionComponent):
    '''Cross section characteristics needed for tensile specimens
    '''

    matrix_cs = WeakRef(MatrixCrossSection, transient=True)

    def __getstate__ (self):
        '''Overriding __getstate__ because of WeakRef usage
        '''
        state = super(ReinfLayoutComponent, self).__getstate__()

        for key in ['matrix_cs', 'matrix_cs_']:
            if key in state:
                del state[ key ]

        return state

    #===============================================================================
    # Plotting functions
    #===============================================================================

    def plot_geometry(self, ax, clr):
        '''Plot geometry'''
        return

    def plot_eps(self, ax):
        return

    def plot_sig(self, ax):
        return

    def plot(self, fig):
        '''Plots the cross section - particular reinforcement component
        plotted with distinctive color to others
        '''
        ax1 = fig.add_subplot(1, 2, 1)
        self.state.plot_geometry(ax1)
        self.plot_geometry(ax1, clr='red')
        ax2 = fig.add_subplot(1, 2, 2)
        self.material_law_.plot_ax(ax2)



#-------------------------------------------------------------------------------
#
# Copyright (c) 2009, IMB, RWTH Aachen.
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in simvisage/LICENSE.txt and may be redistributed only
# under the conditions described in the aforementioned license.  The license
# is also available online at http://www.simvisage.com/licenses/BSD.txt
#
# Thanks for using Simvisage open source!
#
# Created on Feb 23, 2010 by: rch

from traits.api import \
    HasTraits, Directory, List, Int, Float, Any, \
    on_trait_change, File, Constant, Instance, Trait, \
    Array, Str, Property, cached_property, WeakRef, \
    Dict, Button, Bool, Enum, Event, implements, \
    DelegatesTo

from traitsui.api import \
    View, Item, DirectoryEditor, TabularEditor, HSplit, VGroup, \
    TableEditor, EnumEditor, Handler, FileEditor, VSplit, Group, \
    InstanceEditor, HGroup, Spring

# # overload the 'get_label' method from 'Item' to display units in the label
from util.traits.ui.item import \
    Item

from matresdev.db.simdb.simdb_class import \
    SimDBClass, SimDBClassExt

from math import \
    cos, sin, pi

class FabricLayOut(SimDBClass):
    '''Comprises the characteristics of the textile reinforcement
    '''
    # cross sectional area of the reinforcement [mm^2/m]:
    # in 0 and 90 degree orientation:
    a_tex_0 = Float(unit='mm^2/m', simdb=True)
    a_tex_90 = Float(unit='mm^2/m', simdb=True)
    E_tex_0 = Float(unit='MPa' , simdb=True)
    E_tex_90 = Float(unit='MPa' , simdb=True)

    # spacing of the textile mesh [mm]:
    s_tex_0 = Float(unit='mm', simdb=True)
    s_tex_90 = Float(unit='mm', simdb=True)

    def get_E_tex(self, angle_degree):
        angle_rad = pi / 180. * angle_degree
        return self.E_tex_0 * cos(angle_rad) + self.E_tex_90 * sin(angle_rad)

    # view:
    traits_view = View(
                      Item('key'     , style='readonly'),
                      Item('a_tex_0' , style='readonly', format_str="%.2f"),
                      Item('a_tex_90', style='readonly', format_str="%.2f"),
                      Item('E_tex_0' , style='readonly', format_str="%.0f"),
                      Item('E_tex_90', style='readonly', format_str="%.0f"),
                      Item('s_tex_0' , style='readonly', format_str="%.1f"),
                      Item('s_tex_90', style='readonly', format_str="%.1f"),
                      resizable=True,
                      scrollable=True
                      )

# Setup the database class extension
#
FabricLayOut.db = SimDBClassExt(
            klass=FabricLayOut,

            constants={
               'unreinforced' : FabricLayOut(
                                           a_tex_0=0.,
                                           a_tex_90=0.,
                                           E_tex_0=0.,
                                           E_tex_90=0.,
                                           s_tex_0=1.,
                                           s_tex_90=1.,
                                           ),

               # AR-glas textile (2400 tex) compact binding (Franse):
               # new textile tag "2D-03-08" corresponds to "MAG-07-03"
               #
               'MAG-07-03' : FabricLayOut(
                                           a_tex_0=107.89,
                                           a_tex_90=106.61,
#                                           E_tex_0=72000.,
#                                           E_tex_90=72000.,
                                           E_tex_0=66831.,  # (l=500mm;2400tex;200mm/min)
                                           E_tex_90=66831.,  # (l=500mm;2400tex;200mm/min)
                                           s_tex_0=8.3,
                                           s_tex_90=8.4,
                                           ),

               # AR-glas textile (2400 tex) tricot binding:
               #
               '2D-15-10' : FabricLayOut(
                                           a_tex_0=107.89,
                                           a_tex_90=106.61,
#                                           E_tex_0=72000.,
#                                           E_tex_90=72000.,
                                           E_tex_0=66831.,  # (l=500mm;2400tex;200mm/min)
                                           E_tex_90=66831.,  # (l=500mm;2400tex;200mm/min)
                                           s_tex_0=8.3,
                                           s_tex_90=8.4,
                                           ),


               # AR-glas textile (2 x 1200 tex in 0-direction):
               #
               '2D-02-06a' : FabricLayOut(
                                           a_tex_0=71.65,
                                           a_tex_90=53.31,
#                                           E_tex_0=72000.,
#                                           E_tex_90=72000.,
                                           E_tex_0=66831.,  # (l=500mm;2400tex;200mm/min)
                                           E_tex_90=66831.,  # (l=500mm;2400tex;200mm/min)
                                           s_tex_0=12.5,
                                           s_tex_90=8.4,
                                           ),

               # carbon textile / tricot binding ("Trikot")
               # 2 x 800 tex in 0-direction (2v1l): spacing 12,5 mm
               # 1 x 800 tex in 90-direction (1v1l): effective spacing of 7.7 mm!
               #
               '2D-14-10' : FabricLayOut(
                                           a_tex_0=73.89,
                                           a_tex_90=58.,  # 53.9 * 8.4mm / 7.7mm
                                           E_tex_0=180862.,  # stiffness value taken from yarn tests 1600tex, l=300mmm;
                                           E_tex_90=180862.,  # stiffness value taken from yarn tests 1600tex, l=300mmm;
                                           s_tex_0=12.5,
                                           s_tex_90=5.83,  # input for manufacturing machine was 8.4 mm!
                                           ),

               # carbon textile / tricot binding ("Trikot")
               # 1 x 800 tex in 0-direction (1v1l): 8.3 mm
               # 1 x 800 tex in 90-direction (1v1l): effective spacing of 7.7 mm!
               #
               '2D-04-11' : FabricLayOut(
                                           a_tex_0=53.9,
                                           a_tex_90=58.,  # 53.9 * 8.4mm / 7.7mm
                                           E_tex_0=180862.,  # stiffness value taken from yarn tests 1600tex, l=300mmm;# yarn tests with l=125mm: E=165GPa
                                           E_tex_90=180862.,  # stiffness value taken from yarn tests 1600tex, l=300mmm;
                                           s_tex_0=8.3,
                                           s_tex_90=7.7,  # input for manufacturing machine was 8.4 mm!
                                           ),

               # carbon textile / tricot binding ("Trikot") with defect
               # due to the manifacturing process (rovings are separated by the binding thread)
               # 1 x 800 tex in 0-direction (1v1l): 8.3 mm
               # 1 x 800 tex in 90-direction (1v1l): effective spacing of 7.7 mm!
               #
               '2D-04-11_defect' : FabricLayOut(
                                           a_tex_0=53.9,
                                           a_tex_90=58.,  # 53.9 * 8.4mm / 7.7mm
                                           E_tex_0=180862.,  # stiffness value taken from yarn tests 1600tex, l=300mmm;# yarn tests with l=125mm: E=165GPa
                                           E_tex_90=180862.,  # stiffness value taken from yarn tests 1600tex, l=300mmm;
                                           s_tex_0=8.3,
                                           s_tex_90=7.7,  # input for manufacturing machine was 8.4 mm!
                                           ),

               # demonstrator textile (carbon)
               # carbon textile / tissue binding ("Tuch")
               # 1 x 800 tex in 0-direction (1v1l): 8.3 mm
               # 1 x 800 tex in 90-direction (1v1l): effective spacing of 7.7 mm!
               #
               '2D-05-11' : FabricLayOut(
                                           a_tex_0=53.9,
#                                           a_tex_0 = 55.4,
                                           a_tex_90=58.,  # 53.9 * 8.4mm / 7.7mm
#                                           a_tex_90=60.,  # 55.0 * 8.4mm / 7.7mm
                                           E_tex_0=180862.,  # stiffness value taken from yarn tests 1600tex, l=300mmm;# yarn tests with l=125mm: E=165GPa
                                           E_tex_90=180862.,  # stiffness value taken from yarn tests 1600tex, l=300mmm;
                                           s_tex_0=8.3,
                                           s_tex_90=7.7,  # input for manufacturing machine was 8.4 mm!
                                           ),

               # AR-glass tissue binding (barrel shell)
               # tissue binding ("Tuch")
               # 1 x 1200 tex in 0-direction (1v1l): 8.3 mm
               # 1 x 1200 tex in 90-direction (1v1l): effective spacing of 7.7 mm!
               #
               '2D-09-12' : FabricLayOut(
                                           a_tex_0=54.0,
#                                           a_tex_0=55.4,
                                           a_tex_90=58.2,  # 0.448mm2 / 7.7mm
#                                           a_tex_90=60.,  # 55.0 * 8.4mm / 7.7mm
#                                           E_tex_0=72000.,
#                                           E_tex_90=72000.,
                                           E_tex_0=66831.,  # (l=500mm;2400tex;200mm/min)
                                           E_tex_90=66831.,  # (l=500mm;2400tex;200mm/min)
                                           s_tex_0=8.3,
                                           s_tex_90=7.7,  # input for manufacturing machine was 8.4 mm!
                                           ),


               # carbon textile (heavy tow 3300 tex): Trikot binding
               # in SFB-yarn tensile test: sig_max = 1020 MPa, eps_max = 10,3E-3
               #
               '2D-18-10' : FabricLayOut(
                                           a_tex_0=76.96,
                                           a_tex_90=123,
                                           E_tex_0=180862.,  # stiffness value taken from yarn tests 1600tex, l=300mmm; # yarn tests with l=125mm: E=107500 MPa
                                           E_tex_90=180862.,
                                           s_tex_0=24.0,
                                           s_tex_90=15.0,
                                           ),

               # carbon with epoxid rasin:
               # 3300 tex
               #
               'C-Grid-C50' : FabricLayOut(
                                           a_tex_0=40.0,
                                           a_tex_90=44.9,
                                           E_tex_0=234500.,
                                           E_tex_90=234500.,
                                           s_tex_0=46.0,
                                           s_tex_90=41.0,
                                           ),

               # EP coated carbon textile
               # 2 x 24 K in 0-direction
               # 2 x 24 K in 90-direction
               #
               'C-Grid-C50-25' : FabricLayOut(
                                           a_tex_0=74.,
                                           a_tex_90=74.,
                                           E_tex_0=234500.,
                                           E_tex_90=234500.,
                                           s_tex_0=2.5,
                                           s_tex_90=2.5,
                                           ),

               # SBR coated carbon textile
               # 1 x 50K in 0-direction
               # 1 x 50K in 90-direction
               #
               'Grid-600' : FabricLayOut(
                                           a_tex_0=170.,
                                           a_tex_90=102.,
                                           E_tex_0=180000.,  # stiffness value taken from yarn tests #165000.,
                                           E_tex_90=180000.,
                                           s_tex_0=10.8,
                                           s_tex_90=18.,
                                           ),

               # SBR coated carbon textile
               # 2 x 24K in 0-direction
               # 2 x 24K in 90-direction
               #
               'FRA-CAR/SB' : FabricLayOut(
                                           a_tex_0=55.,
                                           a_tex_90=46.,
                                           E_tex_0=1.,
                                           E_tex_90=1.,
                                           s_tex_0=2.3,
                                           s_tex_90=2.1,
                                           ),

               # EP coated AR-glas textile
               #
               'FRA-AR/EP' : FabricLayOut(
                                           a_tex_0=107.,
                                           a_tex_90=134.,
                                           E_tex_0=1.,
                                           E_tex_90=1.,
                                           s_tex_0=2.0,
                                           s_tex_90=1.6,  # average spacing for alternating arrangement of rovings (3/1/1)
                                           ),

               # SBR coated carbon textile
               # 1 x 12K (800tex) in 0-direction
               # ? x ? K in 90-direction
               #
               'CAR-800-SBR_TUD' : FabricLayOut(
                                           a_tex_0=62.8,
                                           a_tex_90=1.,
                                           E_tex_0=245000.,  # SBR coating 800 tex
                                           E_tex_90=245000.,
                                           s_tex_0=7.1,
                                           s_tex_90=1.,
                                           ),

               # SBR coated carbon textile
               # 12K (800tex) rovings with SBR-coating
               # A_rov = 0.45m^2
               #
               'NWM3-016-09-b1' : FabricLayOut(
                                           a_tex_0=62.5,  # = 0.45 m^2 / 0.0072 m
                                           a_tex_90=31.3,  # = 0.45 m^2 / 0.0144 m
                                           E_tex_0=245000.,  # SBR coating 800 tex
                                           E_tex_90=245000.,
                                           s_tex_0=7.2,
                                           s_tex_90=14.4,
                                           ),

               # SBR coated carbon textile
               # 50K (3300tex) rovings with SBR-coating 0-direction
               # 12L (800tex) rovings with SBR-coating in 90-direction
               # A_rov = 1.84m^2
               #
               'CAR-3300-SBR_BTZ2' : FabricLayOut(
                                           a_tex_0=144.9,  # = 1.84 m^2 / 0.0127 m
                                           a_tex_90=25.0,  # = 0.45 m^2 / 0.018 m
                                           E_tex_0=170000.,  # yarn test with SBR coating (3300tex)
                                           E_tex_90=152000.,  # yarn test with SBR coating (800tex)
                                           s_tex_0=12.7,
                                           s_tex_90=18.0,
                                           ),

               # epoxy resin coated carbon textile
               # 50K (3300tex) rovings with EP-coating 0-direction and 90-direction
               # A_rov = 1.84m^2
               #
               'CAR-3300-EP_Q90' : FabricLayOut(
                                           a_tex_0=90.,  # = 1.84 m^2 / 0.02 m
                                           a_tex_90=90,
                                           E_tex_0=245000.,  # yarn test with EP impregnation (3300tex)
                                           E_tex_90=245000.,
                                           s_tex_0=21.,
                                           s_tex_90=21.,
                                           ),
#
               'Q85/85-CCE-21' : FabricLayOut(
                                           a_tex_0=85.,  # = 1.81 m^2 / 0.021 m
                                           a_tex_90=85,
                                           E_tex_0=246100.,  # Modulus from single yarn test data base
                                           E_tex_90=246100., 
                                           s_tex_0=21.,
                                           s_tex_90=21.,
                                           ),
#
               'Q95/95-CCE-38' : FabricLayOut(
                                           a_tex_0=95.,  # = 3.62 m^2 / 0.038 m
                                           a_tex_90=95,
                                           E_tex_0=246100.,  # Modulus from single yarn test data base
                                           E_tex_90=246100., 
                                           s_tex_0=38.,
                                           s_tex_90=38.,
                                           ),  
#
               'Q142/142-CCE-25' : FabricLayOut(
                                           a_tex_0=142,  # = 3.62 m^2 / 0.025 m
                                           a_tex_90=142,
                                           E_tex_0=246100.,  # Modulus from single yarn test data base
                                           E_tex_90=246100., 
                                           s_tex_0=25.,
                                           s_tex_90=25.,
                                           ), 
#
               'Q142/142-CCE-38' : FabricLayOut(
                                           a_tex_0=142,  # = 5.42 m^2 / 0.038 m
                                           a_tex_90=142,
                                           E_tex_0=246100.,  # Modulus from single yarn test data base
                                           E_tex_90=246100., 
                                           s_tex_0=38.,
                                           s_tex_90=38.,
                                           ), 
#
               'Q145/145-AAE-25' : FabricLayOut(
                                           a_tex_0=145,  # = 3.69 m^2 / 0.025 m
                                           a_tex_90=145,
                                           E_tex_0=65000.,  # ToDo: Verify Modulus by comparison to roving tests
                                           E_tex_90=65000., # ToDo: Verify Modulus by comparison to roving tests
                                           s_tex_0=25.,
                                           s_tex_90=25.,
                                           ), 
#
               'Q121/121-AAE-38' : FabricLayOut(
                                           a_tex_0=121,  # = 4.62 m^2 / 0.038 m
                                           a_tex_90=121,
                                           E_tex_0=65000.,  # ToDo: Verify Modulus by comparison to roving tests
                                           E_tex_90=65000., # ToDo: Verify Modulus by comparison to roving tests
                                           s_tex_0=38.,
                                           s_tex_90=38.,
                                           ),    
#
               'Q97/97-AAE-38' : FabricLayOut(
                                           a_tex_0=97,  # = 3.69 m^2 / 0.038 m
                                           a_tex_90=97,
                                           E_tex_0=65000.,  # ToDo: Verify Modulus by comparison to roving tests
                                           E_tex_90=65000., # ToDo: Verify Modulus by comparison to roving tests
                                           s_tex_0=38.,
                                           s_tex_90=38.,
                                           ),
#
               'Q87/87-AAE-21' : FabricLayOut(
                                           a_tex_0=87,  # = 1.85 m^2 / 0.025 m
                                           a_tex_90=87,
                                           E_tex_0=65000.,  # ToDo: Verify Modulus by comparison to roving tests
                                           E_tex_90=65000., # ToDo: Verify Modulus by comparison to roving tests
                                           s_tex_0=21.,
                                           s_tex_90=21.,
                                           ),
#
               'Q87/87-AAS-21' : FabricLayOut(
                                           a_tex_0=87,  # = 1.85 m^2 / 0.025 m
                                           a_tex_90=87,
                                           E_tex_0=65000.,  # ToDo: Verify Modulus by comparison to roving tests
                                           E_tex_90=65000., # ToDo: Verify Modulus by comparison to roving tests
                                           s_tex_0=21.,
                                           s_tex_90=21.,
                                           ), 
#
               'Q142/142-CCS-25' : FabricLayOut(
                                           a_tex_0=142,  # = 3.62 m^2 / 0.025 m
                                           a_tex_90=142,
                                           E_tex_0=65000.,  # ToDo: Verify Modulus by comparison to roving tests
                                           E_tex_90=65000., # ToDo: Verify Modulus by comparison to roving tests
                                           s_tex_0=25.,
                                           s_tex_90=25.,
                                           ),
#
               'R106/29-CGS-17x31' : FabricLayOut(
                                           a_tex_0=106,  # = 1.81 m^2 / 0.017 m
                                           a_tex_90=29,
                                           E_tex_0=245000.,  # ToDo: Verify Modulus by comparison to roving tests
                                           E_tex_90=65000., # ToDo: Verify Modulus by comparison to roving tests
                                           s_tex_0=17.,
                                           s_tex_90=31.,
                                           ),
#                            
               'CAR-6600-SBR_E0003' : FabricLayOut(
                                           a_tex_0=141.,  # = 3.67 m^2 / 0.026 m
                                           a_tex_90=141,
                                           E_tex_0=245000.,  # ToDo: Verify Modulus by comparison to roving tests
                                           E_tex_90=245000., # ToDo: Verify Modulus by comparison to roving tests
                                           s_tex_0=26.,
                                           s_tex_90=26.,
                                           ),    
               # styrol-butadiene impregnated Textile, Manufacturer: V.Fraas
               # 100K (6600tex) rovings in 0-direction and 90-direction
               # A_rov = 3.67mm^2
               #                                                                                                                              
             }
            )

if __name__ == '__main__':
    FabricLayOut.db.configure_traits()



'''
Created on Aug 23, 2012

@author: rch
'''

from traits.api import \
    HasStrictTraits, Property, WeakRef, \
    cached_property, on_trait_change, List

from traitsui.api import \
    View, Item

import numpy as np

from bmcs_beam.mxn.mfn import MFnLineArray

import weakref


class CLBase(HasStrictTraits):

    '''Base class for Effective Crack Bridge Laws.'''

    def __init__(self, *args, **kw):
        super(HasStrictTraits, self).__init__(*args, **kw)

    def set_cparams(self, *args):
        for name, value in zip(self.cnames, args):
            setattr(self, name, value)

    arr = Property()

    def _get_arr(self):
        return self.eps_arr, self.sig_arr

    mfn = Property()

    def _get_mfn(self):
        return MFnLineArray(xdata=self.eps_arr,
                            ydata=self.sig_arr)

    mfn_vct = Property()

    def _get_mfn_vct(self):
        return np.vectorize(self.mfn.get_value, otypes=[np.float])

    def plot(self, fig):
        ax = fig.add_subplot(1, 1, 1)
        self.plot_ax(ax)

    def plot_ax(self, ax):
        ax.plot(*self.arr)

    def plot_custom(self, ax, color='blue', linestyle='-', linewidth=2, label='<unnamed>'):
        ax.plot(
            *self.arr, lw=linewidth, color=color, ls=linestyle, label=label)

    #=========================================================================
    # Management of backward links
    #=========================================================================

    state_link_lst = List(transient=True)
    '''List of backward links to objects using the fabric
    '''

    def _state_link_lst_default(self):
        return []

    @on_trait_change('+input')
    def notify_change(self):
        for link in self.state_link_lst:
            if link():
                link().law_changed = True

    def add_link(self, link_to_add):
        '''Adding a backward link to the list - to be called
        from objects using the law
        '''
        if link_to_add not in self.state_link_lst:
            self.state_link_lst.append(weakref.ref(link_to_add))

    def del_link(self, link_to_del):
        '''Removing a backward link from the list - to be called
        from objects using the law
        '''
        self.state_link_lst[:] = [
            link for link in self.state_link_lst if link() != link_to_del]

    def default_traits_view(self):

        input_traits = self.traits(input=lambda x: x != None)

        citems = [Item(name) for name in input_traits]
        return View(*citems,
                    buttons=['OK', 'Cancel']
                    )




import bmcs_utils.api as bu
import sympy as sp
import numpy as np
from scipy.integrate import cumtrapz

class TimeDependentCrackingExpr(bu.SymbExpr):

    t = sp.symbols('t', nonnegative=True)

    T_m = sp.Symbol("T_m", positive = True)
    T_s = sp.Symbol("T_s", positive = True)

    omega_fn = 1 - sp.exp(-(t / T_s) ** T_m)

    T_prime_0 = sp.Symbol("T_prime_0", positive=True)

    T_t = (1 - omega_fn) * T_prime_0 * t

    T_prime_t = sp.simplify(T_t.diff(t))

    t_argmax_T = sp.Symbol("t_argmax_T")
    T_s_sol = sp.solve(sp.Eq(sp.solve(T_prime_t, t)[0], t_argmax_T), T_s)[0]

    T_max = sp.Symbol("T_max", positive=True)
    T_prime_0_sol = sp.solve(sp.Eq(T_t.subs(T_s, T_s_sol).subs(t, t_argmax_T), T_max),
                             T_prime_0)[0]

    T_max_t = sp.simplify(T_t.subs({T_s: T_s_sol, T_prime_0: T_prime_0_sol}))
    dot_T_max_t = sp.simplify(T_max_t.diff(t))

    ## Time dependent compressive strength

    s = sp.Symbol("s", positive=True)

    beta_cc = sp.exp(s * (1 - sp.sqrt(28 / t)))

    f_cm_28 = sp.Symbol("f_cm28", positive=True)

    f_cm_t = beta_cc * f_cm_28

    f_ctm = sp.Symbol("f_ctm", positive=True)
    alpha_f = sp.Symbol("alpha_f", positive=True)
    f_ctm_t = beta_cc * f_ctm

    E_cm_28 = sp.Symbol("E_cm28", positive=True)
    E_cm_t = (f_cm_t / f_cm_28) ** 0.3 * E_cm_28
    dot_E_cm_t = E_cm_t.diff(t)

    alpha = sp.Symbol("alpha", positive=True)
    eps_eff_t = -alpha * T_max_t
    dot_eps_eff_t = -alpha * dot_T_max_t

    sig_t = E_cm_t * eps_eff_t
    dot_sig_t = sp.simplify(E_cm_t * dot_eps_eff_t) #  + dot_E_cm_t * eps_eff_t)

    symb_model_params = ['T_max', 't_argmax_T', 'T_m',
                         's', 'f_cm_28', 'f_ctm', 'alpha_f',
                         'E_cm_28', 'alpha']

    symb_expressions = [
        ('T_max_t', ('t',)),
        ('f_cm_t', ('t',)),
        ('f_ctm_t', ('t',)),
        ('E_cm_t', ('t',)),
        ('sig_t', ('t',)),
        ('dot_sig_t', ('t',))
    ]

class TimeDependentCracking(bu.InteractiveModel,bu.InjectSymbExpr):

    name = 'Time Dependent Cracking'

    symb_class = TimeDependentCrackingExpr

    T_m = bu.Float(1)
    T_max = bu.Float(45)
    t_argmax_T = bu.Float(1.4)
    s = bu.Float(0.2)
    f_cm_28 = bu.Float(30)
    f_ctm = bu.Float(4)
    alpha_f = bu.Float(1)
    E_cm_28 = bu.Float(28000)
    alpha = bu.Float(1e-5)
    t_max = bu.Float(10)

    ipw_view = bu.View(
        bu.Item('T_max', latex=r'T_{\max}',
                editor=bu.FloatRangeEditor(low=5,high=100,
                                           continuous_update=True)),
        bu.Item('t_argmax_T', latex='\mathrm{argmax} T',
                editor=bu.FloatRangeEditor(low=0.4,high=3,
                                           continuous_update=True)),
        bu.Item('T_m', latex='T_m',
                editor=bu.FloatRangeEditor(low=0.1,high=3,
                                           continuous_update=True)),
        bu.Item('s', latex='s'),
#        bu.Item('f_cm_28', latex=r'f_\mathrm{cm}^{28}'),
        bu.Item('f_ctm', latex=r'f_\mathrm{ctm}',
                editor=bu.FloatRangeEditor(low=3,high=10,
                                           continuous_update=True)),
        bu.Item('alpha_f', latex=r'\alpha_f'),
        bu.Item('E_cm_28', latex=r'E_\mathrm{cm}^{28}',
                editor=bu.FloatRangeEditor(low=25000,high=40000,
                                           continuous_update=True)),
        bu.Item('alpha', latex=r'\alpha'),
    )

    def subplots(self, fig):
        ax_sig = fig.subplots(1,1)
        ax_T = ax_sig.twinx()
        return ax_sig, ax_T

    def update_plot(self, axes):
        ax_sig, ax_T = axes
        t_range = np.linspace(1e-4,self.t_max,100)

        dot_sig_range = self.symb.get_dot_sig_t(t_range)
        sig2_range = cumtrapz(dot_sig_range,t_range,initial=0)
        ax_sig.plot(t_range, sig2_range, lw=2, color='green', label='$\sigma$')
        ax_sig.fill_between(t_range, sig2_range, 0, color='green', alpha=0.1)
        ax_sig.set_xlabel('$t$ [days]')
        ax_sig.set_ylabel('$\sigma$ [MPa]')

        f_ctm_range = self.symb.get_f_ctm_t(t_range)
        ax_sig.plot(t_range, f_ctm_range, lw=2, color='red', label='$f_\mathrm{ctm}$')
        ax_sig.fill_between(t_range, f_ctm_range, 0, color='red', alpha=0.05)
        ax_sig.legend()

        T_range = self.symb.get_T_max_t(t_range)
        ax_T.plot(t_range, T_range, color='blue',label='temperature')
        ax_T.fill_between(t_range, T_range, 0, color='blue', alpha=0.05)
        ax_T.set_ylabel('$T$ [C$^o$]')
        ax_T.legend()


'''
Created on 29. 6. 2014

@author: Vancik
'''

from traits.api import \
    Instance, Button, Str, Trait

from traitsui.api import \
    View, Item, HGroup, Handler, spring, UIInfo

class MaterialTypeHandler(Handler):
    '''
    '''

    # The UIInfo object associated with the view:
    info = Instance(UIInfo)

    ok = Button('OK')
    delete = Button('OK')
    cancel = Button('Cancel')

    law_name = Str

    # The customization view:
    new_view = View(
        HGroup(
            spring,
            Item('law_name'),
            Item('law_type'),
            Item('ok', show_label=False),
            Item('cancel', show_label=False),
        ),
        kind='live'
    )

    del_view = View(
        HGroup(
            spring,
            Item(name='', label='Really delete?'),
            Item('delete', show_label=False),
            Item('cancel', show_label=False),
        ),
        kind='popup'
    )

    # Event handlers:
    def object_new_law_changed (self, info):
        if info.initialized:
            self.info = info
            laws = info.object.possible_laws
            self.add_trait('law_type', Trait(list(laws.values())[0], laws))
            self._ui = self.edit_traits(parent=info.new_law.control, view='new_view')

    def object_del_law_changed (self, info):
        if info.initialized:
            self.info = info
            self._ui = self.edit_traits(parent=info.del_law.control, view='del_view')

    def _ok_fired (self):
        object = self.info.object
        if self.law_name == '':
            print('Please enter law name!')
        elif object.mtrl_laws.get(self.law_name, None):
            print('Law name already occupied!')
        else:
            object.mtrl_laws[self.law_name] = self.law_type_()
            self._ui.dispose()
            self.info.ui.parent.updated = True

    def _delete_fired (self):
        object = self.info.object
        del object.mtrl_laws[object.chosen_law]
        self._ui.dispose()
        self.info.ui.parent.updated = True

    def _cancel_fired (self):
        self._ui.dispose()



'''
Created on Sep 4, 2012

@todo: introduce the dock feature for the views
@todo: classify the state changes and provide examples.


@author: rch
'''

from traits.api import \
    HasStrictTraits, Float, Property, cached_property, Int, \
    Trait, Event, on_trait_change, Instance, Button, Callable, \
    DelegatesTo, Constant, List

from matplotlib.figure import \
    Figure

from traitsui.api import \
    TreeEditor, TreeNode, View, Item, Group, HSplit, VGroup, HGroup

from bmcs_beam.mxn.mxn_tree_node import \
    ReinfLayoutTreeNode

from bmcs_beam.mxn.cross_section_state import \
    CrossSectionState

from bmcs_beam.mxn.matrix_cross_section import \
    MatrixCrossSection

from bmcs_beam.mxn.reinf_layout import \
    ReinfLayoutComponent

import numpy as np


class CrossSection(CrossSectionState):
    '''Cross section characteristics needed for tensile specimens
    '''
    matrix_cs = Instance(MatrixCrossSection)
    def _matrix_cs_default(self):
        return MatrixCrossSection(material='default_mixture')

    reinf = List(Instance(ReinfLayoutComponent))
    '''Components of the cross section including the matrix and reinforcement.
    '''

    matrix_cs_with_state = Property(depends_on='matrix_cs')
    @cached_property
    def _get_matrix_cs_with_state(self):
        self.matrix_cs.state = self
        return self.matrix_cs

    reinf_components_with_state = Property(depends_on='reinf')
    '''Components linked to the strain state of the cross section
    '''
    @cached_property
    def _get_reinf_components_with_state(self):
        for r in self.reinf:
            r.state = self
            r.matrix_cs = self.matrix_cs
        return self.reinf

    unit_conversion_factor = Constant(1000.0)

    '''Convert the MN to kN
    '''

    #===========================================================================
    # State management
    #===========================================================================

    notify_change_ext = Callable(None, transient=True)
    '''Notifier of component changes for external clients
    '''

    changed = Event
    '''Notifier of a change in some component of a cross section
    '''

    @on_trait_change('+eps_input')
    def _notify_eps_change(self):
        self.matrix_cs.eps_changed = True
        for c in self.reinf:
            c.eps_changed = True

    @on_trait_change('changed')
    def _notify_component_change(self):
        if self.notify_change_ext != None:
            self.notify_change_ext()

    #===========================================================================
    # Cross-sectional stress resultants
    #===========================================================================

    N = Property(depends_on='changed,+eps_input')
    '''Get the resulting normal force.
    '''
    @cached_property
    def _get_N(self):
        N_matrix = self.matrix_cs_with_state.N
        return N_matrix + np.sum([c.N for c in self.reinf_components_with_state])

    M = Property(depends_on='changed,+eps_input')
    '''Get the resulting moment.
    '''
    @cached_property
    def _get_M(self):
        M_matrix = self.matrix_cs_with_state.M
        M = M_matrix + np.sum([c.M for c in self.reinf_components_with_state])
        return M - self.N * self.matrix_cs.geo.gravity_centre

    #===============================================================================
    # Plotting functions
    #===============================================================================

    def plot_geometry(self, ax):
        self.matrix_cs_with_state.geo.plot_geometry(ax)
        for r in self.reinf_components_with_state:
            r.plot_geometry(ax)

    def plot_eps(self, ax):
        self.matrix_cs_with_state.plot_eps(ax)
        for r in self.reinf_components_with_state:
            r.plot_eps(ax)
        ax.spines['left'].set_position('zero')
        ax.spines['right'].set_color('none')
        ax.spines['top'].set_color('none')
        ax.spines['left'].set_smart_bounds(True)
        ax.spines['bottom'].set_smart_bounds(True)
        ax.xaxis.set_ticks_position('bottom')
        ax.yaxis.set_ticks_position('left')

    def plot_sig(self, ax):
        self.matrix_cs_with_state.plot_sig(ax)
        for r in self.reinf_components_with_state:
            r.plot_sig(ax)
        ax.spines['left'].set_position('zero')
        ax.spines['right'].set_color('none')
        ax.spines['top'].set_color('none')
        ax.spines['left'].set_smart_bounds(True)
        ax.spines['bottom'].set_smart_bounds(True)
        ax.xaxis.set_ticks_position('bottom')
        ax.yaxis.set_ticks_position('left')

    def plot(self, fig):
        ax1 = fig.add_subplot(1, 3, 1)
        ax2 = fig.add_subplot(1, 3, 2)
        ax3 = fig.add_subplot(1, 3, 3)
        self.plot_geometry(ax1)
        self.plot_eps(ax2)
        self.plot_sig(ax3)

    #===========================================================================
    # Visualisation related attributes
    #===========================================================================

    node_name = 'Cross section'

    tree_node_list = Property
    @cached_property
    def _get_tree_node_list(self):
        return [self.matrix_cs_with_state,
                ReinfLayoutTreeNode(cs_state=self)]

    tree_view = View(VGroup(Item('eps_up'),
                       Item('eps_lo'),
                       label='Cross section'
                      ),
                resizable=True,
                buttons=['OK', 'Cancel']
                )

    traits_view = View(VGroup(Item('eps_up'),
                       Item('eps_lo'),
                       Item('matrix_cs'),
                       label='Cross section'
                      ),
                resizable=True,
                buttons=['OK', 'Cancel']
                )

if __name__ == '__main__':
    cs = CrossSection()
    cs.configure_traits()


# Import Standard Libraries
import logging
import scipy as np

# Import Local Libraries
from Utilities import *

#===========================================================================
#   EC2 Equations - Material properties
#===========================================================================


def elastic_modulus(fck, units="MPa"):
    """ Input:  fck = char. comp. strength of concrete
                units = "MPa" or "psi" (default = "MPa")
        Output: Ec = mean elastic modulus of concrete """
    fck = convert_2_MPa(fck, units)
    fcm = fck+8
    Ec = 22000*(fcm/10)**0.3
    return Ec if units == "MPa" else convert_2_psi(Ec, "MPa")


def tensile_strength(fck, units="MPa"):
    """ Input:  fck = char. comp. strength of concrete
                units = "MPa" or "psi" (default = "MPa")
        Output: fctm = mean tensile strength of concrete """
    fck = convert_2_MPa(fck, units)
    fcm = fck+8
    fctm = 0.3*fck**(2/3) if fck <= 50 else 2.12*np.log(1+fcm/10)
    return fctm if units == "MPa" else convert_2_psi(fctm, "MPa")


def flex_tensile_strength(fck, h, units="MPa"):
    """ Input:  fck = char. comp. strength of concrete
                h = height of reinforced concrete beam
                units = "MPa" or "psi" (default = "MPa") 
        Output: fctm,fl = mean tensile strength for flexure """
    fck = convert_2_MPa(fck, units)
    fctm = tensile_strength(fck)
    h = convert_2_mm(h, units)
    fctm = min((1.6-h/1000)*fctm, fctm)
    return fctm if units == "MPa" else convert_2_psi(fctm, "MPa")


def ultimate_strain(fck, units="MPa"):
    """ Input:  fck = char. comp. strength of concrete
                units = "MPa" or "psi" (default = "MPa") 
        Output: ecu3 = ultimate tensile strain """
    fck = convert_2_MPa(fck, units)
    ecu3 = 2.6+35*((90-fck)/100)**4
    return min(ecu3, 3.5)/1000

#===========================================================================
#   EC2 Equations - Parameters
#===========================================================================


def alpha_beta(fck, units="MPa"):
    """ Input:  fck = char. comp. strength of concrete
                units = "MPa" or "psi" (default = "MPa") 
        Output: alpha = factor for bilinear stress block
                beta = (dist. from comp. to Nc)/Xu """
    fck = convert_2_MPa(fck, units)
    alpha = np.ceil((9E-05*fck**2 - 0.0177*fck + 1.4032)*100)/100
    beta = np.ceil((4E-05*fck**2 - 0.0071*fck + 0.634)*100)/100
    return [min(alpha, 0.75), min(beta, 0.39)]


def lambda_eta(fck, units="MPa"):
    """ Input:  fck = char. comp. strength of concrete
                units = "MPa" or "psi" (default = "MPa") 
        Output: la = (height of compressive zone)/Xu
                eta = factor for "Whitney" stress block """
    fck = convert_2_MPa(fck, units)
    la = min(0.8-(fck-50)/400, 0.8)
    eta = min(1-(fck-50)/200, 1.0)
    return [la, eta]

#===========================================================================
#   EC2 Equations - Maximum reinforcement (Ductility)
#===========================================================================


def ductility_requirement(Xu, d, fck, fyd, units="MPa"):
    """ Input:  Xu = dist. from comp. to neutral axis 
                d = dist. from comp. to reinforcement
                fck = char. comp. strength of concrete
                fyd = design steel yield stress
                units = "MPa" or "psi" (default = "MPa")
        Output: Xu_max = Max. dist. to neutral axis """
    [fck, fyd] = convert_2_MPa(np.array([fck, fyd]), units)
    
    ecu = ultimate_strain(fck)  # units="MPa"
    Xu_max = min(ecu*10**6/(ecu*10**6+7*fyd), 0.535)*d
    if Xu < Xu_max:
        logging.info(
            "    Xu = {:6.2f} < Xu_max = {:6.2f}. OK".format(Xu, Xu_max))
    else:
        logging.info(
            "    Xu = {:6.2f} > Xu_max = {:6.2f}. Not OK".format(Xu, Xu_max))
    return Xu_max

#===========================================================================
#   EC2 Equations - Minimum reinforcement (Md > Mcr)
#===========================================================================


def steel_ratio(As, fck, fyk, b, d, h, Xu, units="MPa"):
    """ Input:  As = area of reinforcement steel
                fck = char. comp. strength of concrete
                fyk = char. yield stress of reinforcement
                b = width of beam portion in compression
                d = dist. from comp. to reinforcement
                h = height of reinforced concrete beam
                Xu = maximum dist. to neutral axis
                units = "MPa" or "psi" (default = "MPa")
        Output: A_min = minimum reinforcement area
                A_max = maximum reinforcement area """
    [fck, fyk] = convert_2_MPa(np.array([fck, fyk]), units)
    [b, d, h, Xu] = convert_2_mm(np.array([b, d, h, Xu]), units)
    As = convert_2_mm2(As, units)

    fctm = flex_tensile_strength(fck, h)  # units="MPa"
    A_min = max((0.26*fctm/fyk),0.0013)* (b*d)
    

    fcd = fck/1.5
    fyd = fyk/1.15

    alpha = alpha_beta(fck)[0]  # units="MPa"
    A_max = min(alpha*(fcd/fyd)*b*Xu, 0.4*b*d)

    compare_steel_area(As, A_min, A_max)

    return [A_min, A_max] if units == "MPa" else [A_min/(25.4**2), A_max/(25.4**2)]



from .use_case_container import \
    UseCaseContainer

from .use_case_parametric_study import \
    UCParametricStudy, UCPStudyElementMxN, \
    UCPStudyElementFabricLaw, UCPStudyElementBarLaw, \
    UCPStudyElementMatrixLaw

from .use_case_database import \
    UCDatabase


from .reinf_layout_component import \
    ReinfLayoutComponent

from .reinf_layout_component_tex_layer import \
    RLCTexLayer

from .reinf_layout_component_tex_uniform import \
    RLCTexUniform

from .reinf_layout_component_bar import \
    RLCBar



#-------------------------------------------------------------------------------
#
# Copyright (c) 2013
# IMB, RWTH Aachen University,
# ISM, Brno University of Technology
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in the AramisCDT top directory "license.txt" and may be
# redistributed only under the conditions described in the aforementioned
# license.
#
# Thanks for using Simvisage open source!
#
#-------------------------------------------------------------------------------

import os
import sys
import platform
if platform.system()=='Linux':
    SSHKEY = '~/.ssh/id_rsa'
elif platform.system()=='Windows':
    SSHKEY = r'~\ssh\id_rsa'

try:
    import paramiko
except ImportError as e:
    print("Install package >>paramiko<<.\n%s" % e)

class SFTPServer(object):
    """
    Wraps paramiko for super-simple SFTP uploading and downloading.
    """

    
    def __init__(self, username, password, host, port=22):

        self.transport = paramiko.Transport((host, port))
        privatekeyfile = os.path.expanduser(SSHKEY)
        mykey = paramiko.RSAKey.from_private_key_file(privatekeyfile)
        try:
            if password == '':
                self.transport.connect(username=username, pkey=mykey)
            else:
                self.transport.connect(username=username, password=password)
            self.sftp = paramiko.SFTPClient.from_transport(self.transport)
        except paramiko.ssh_exception.AuthenticationException as e:
            print(e)
            print('Check if you have access to remote server over ssh-key')

    def upload(self, local, remote):
        self.sftp.put(local, remote, self._printTotals)

    def download(self, remote, local):
        self.sftp.get(remote, local, self._printTotals)

    def close(self):
        """
        Close the connection if it's active
        """

        if self.transport.is_active():
            self.sftp.close()
            self.transport.close()

    def _printTotals(self, transferred, toBeTransferred):
        print("Transferred: {0}\tStill to send: {1}\tProcent: {2}".format(transferred, toBeTransferred, transferred / float(toBeTransferred) * 100))
        # sys.stdout.write("\r" + "Downloading... %3f%%" % percent)
        # sys.stdout.flush()

    # with-statement support
    def __enter__(self):
        return self

    def __exit__(self, type, value, tb):
        self.close()





#from sys_matrix import SysSparseMtx, SysDenseMtx
from numpy import array, zeros, arange, array_equal, hstack, dot, sqrt
from scipy.linalg import norm

from os import path
from mathkit.matrix_la.sys_mtx_assembly import SysMtxAssembly
from mathkit.matrix_la.coo_mtx import COOSparseMtx
from mathkit.matrix_la.dense_mtx import DenseMtx
import unittest
from tempfile import mkdtemp

from matresdev.simiter.sim_pstudy import SimPStudy, SimModel

class TestSimArray(unittest.TestCase):
    '''
    Test functionality connected with the application of
    boundary conditions.
    '''
    def setUp(self):

        self.sim_pstudy = SimPStudy( sim_model = SimModel() )    

        
    def test_save( self ):
        '''Calculate a study and save it 
        '''
        res = self.sim_pstudy.sim_array[0,:,:,:]
        print('fraction cached', self.sim_pstudy.sim_array.fraction_cached)
        
        filename = path.join( mkdtemp(), 'foo_file.pst' ) 
        self.sim_pstudy.save( filename )

        self.sim_pstudy.new()
        print('fraction cached', self.sim_pstudy.sim_array.fraction_cached)
        
        self.sim_pstudy.load( filename )
        print('fraction cached', self.sim_pstudy.sim_array.fraction_cached)

        print('FIRST BEFORE', self.sim_pstudy.sim_array.output_table[0])
        self.sim_pstudy.sim_array.clear_cache()
        print('fraction cached', self.sim_pstudy.sim_array.fraction_cached)
        print('FIRST AFTER', self.sim_pstudy.sim_array.output_table[0])
        
        #print res

        #self.assertAlmostEqual( F, -1 )



#-------------------------------------------------------------------------------
#
# Copyright (c) 2009, IMB, RWTH Aachen.
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in simvisage/LICENSE.txt and may be redistributed only
# under the conditions described in the aforementioned license.  The license
# is also available online at http://www.simvisage.com/licenses/BSD.txt
#
# Thanks for using Simvisage open source!
#
# Created on Jan 19, 2010 by: rch

from traits.api import Array, Bool, Enum, Float, HasTraits, \
                                 Instance, Int, Trait, Str, Enum, \
                                 Callable, List, TraitDict, Any, Range, \
                                 Delegate, Event, on_trait_change, Button, \
                                 Interface, implements, Property, cached_property, \
                                 Self, This

from .i_sim_model import ISimModel
from numpy import array
from .sim_output import SimOut
import time

class SimModel( HasTraits ):
    '''
    The SimModel defines an idealization with a input parameters and
    the ranges of validity. There are three types of parameters:

    numerical floating point
    numerical integer
    categorical (enumerators)
    
    The sim_model only names the parameters of the model without specifying
    the possible levels of their evaluation. This is done using two procedures:
    
    get_input_dict
    get_output_dict
    '''
    implements( ISimModel )
    
    param_1 = Float( 3.6, 
                     unit = 'm',
                     ps_levels = ( 3, 5, 4 ) )
    
    param_2 = Float( 8.6, 
                     unit = 'N',
                     ps_levels = ( 2.4, 8.9, 3 ) )
    
    index_1 = Int( 10,
                   ps_levels = ( 2, 4, 2 ) )
    
    material_model = Callable( pstudy = True, transient = True,
                               ps_levels = [ 'matmod_1', 'matmod_2' ] )
    def _material_model_default( self ):
        return self.matmod_1
    
    def matmod_1( self, param_1, param_2 ):
        return param_1 / 2 + param_2

    def matmod_2( self, param_1, param_2 ):
        return param_1 / 2 - param_2
    
    def get_output_1(self):
        return self.param_1**2 + self.param_2 * self.param_1 / self.index_1 \
                * self.material_model( self.param_1, self.param_2 )

    def get_output_2(self):
        return self.param_1 + self.param_2 * self.param_1**2 / self.index_1**3 \
                * self.material_model( self.param_1, self.param_2 )
    
    def get_sim_outputs( self ):
        return [ SimOut( name = '$\sigma_1$', unit = 'kN' ),
                 SimOut( name = 'output_2', unit = 'kNm' ) ]
    
    def peval(self):
        ''' Return the set of outputs for the current setting of parameters.
        '''
        time.sleep(1)
        return array( [self.get_output_1(), 
                       self.get_output_2() ], 
                       dtype = 'float_' )
        
def run():
    from .sim_pstudy import SimPStudy
    import pickle

    model = SimModel()
    model.index_1 = 8
    tp = open('test.pickle', 'w')
    pickle.dump( model, tp )
    tp.close()
    
    tp = open('test.pickle', 'r')
    model = pickle.load( tp )
    tp.close()
    print('model.index_1', model.index_1)
    

    yse = SimArray( model = model )

    
    yse.configure_traits()
    
if __name__ == '__main__':
    run()                     


'''
Created on Apr 9, 2010

Scipt generates a csv-file based on a xls-file containing the test data of a
tensile test with only the time[s], force[kN] and machine displacement[mm] recorded.
The returned .csv-file and .DAT-file allows to read in the test as if it has been recorded 
with 4 displacement gauges (WA_vli, WA_vre, WA_hli, WA_hre). 
The machine displacement starts at an arbitrary value and is reset to start with 0.

@author: alexander
'''

from numpy import \
    array, shape, hstack, savetxt

from os.path import \
    join

from matresdev.db.simdb import \
    SimDB


# name of the tensile tes / xls-file with the machine data
#
TT_name = 'TT-12c-6cm-0-TU-REF1'
#TT_name = 'TT-12c-6cm-0-TU-REF2'
#TT_name = 'TT-12c-6cm-0-TU-WM'
#TT_name = 'TT-12c-6cm-0-TU-WML'
#TT_name = 'TT-12c-6cm-0-TU-WMT'
#TT_name = 'TT-12c-6cm-0-TU-WMTC'

# file path in simdb
#
simdb = SimDB()
file_path = join( simdb.exdata_dir, 'tensile_tests', 'ZiE_2011-08-18_TT-12-6cm-TU_Laminiert' )
file_name_xls = join( file_path, TT_name + '.xls' )

print('read input data from file ' + file_name_xls + '\n')

# open file
#
file = open( file_name_xls, 'r' )

# read heading files without values
#
heading1 = file.readline()
heading2 = file.readline()

# number of columns
#
n_columns = 3

# replace comma with dot
#
file_str = file.read().replace( ",", "." )

# split string into separate values-strings
#
str_list = file_str.split()

# convert string-values into floats and convert to array
#
data_arr_1d = array( [float( val ) for val in str_list] )

# reshape 1d-array into 3-columns-array
#
n_rows = shape( data_arr_1d )[0] / n_columns
data_arr = data_arr_1d.reshape( n_rows, n_columns )

# prepare outputarr duplicating the displacement column 3 times
# and reseting the first value to 0.
# use machine displacement [mm] for displacement gauge ("WA")
WA = ( data_arr[:, 1] - data_arr[0, 1] )[:, None]

# reference time [s]
time = data_arr[:, 0][:, None]

# force [kN]
force = data_arr[:, 2][:, None]

output_arr = hstack( ( time, force, -WA, -WA, -WA, -WA ) )
print('output_arr', output_arr)

file_name_csv = join( file_path, TT_name + '.csv' )

print('save output data to file ' + file_name_csv + '\n')

savetxt( file_name_csv, output_arr, delimiter = ';' )



file_name_DAT = join( file_path, TT_name + '.DAT' )

print('generate .DAT-file ' + file_name_DAT + '\n')

DAT_txt = "#BEGINCHANNELHEADER\n\
200,Bezugskanal\n\
201,Erfassungskanal\n\
202,s (ab Start)\n\
\n\
#BEGINCHANNELHEADER\n\
200,Kraft\n\
201,Erfassungskanal\n\
202,kN\n\
\n\
#BEGINCHANNELHEADER\n\
200,W10_vli\n\
201,Erfassungskanal\n\
202,mm\n\
\n\
#BEGINCHANNELHEADER\n\
200,W10_vre\n\
201,Erfassungskanal\n\
202,mm\n\
\n\
#BEGINCHANNELHEADER\n\
200,W10_hli\n\
201,Erfassungskanal\n\
202,mm\n\
\n\
#BEGINCHANNELHEADER\n\
200,W20_hre\n\
201,Erfassungskanal\n\
202,mm\n\
"
file_DAT = open( file_name_DAT, 'w' )
file_DAT.write( DAT_txt )



#-------------------------------------------------------------------------------
#
# Copyright (c) 2009, IMB, RWTH Aachen.
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in simvisage/LICENSE.txt and may be redistributed only
# under the conditions described in the aforementioned license.  The license
# is also available online at http://www.simvisage.com/licenses/BSD.txt
#
# Thanks for using Simvisage open source!
#ve_study
# Created on Feb 1, 2010 by: rch

""" Application window. """


import os
#os.environ['ETS_TOOLKIT'] = 'qt4'

#
from traits.api import \
    HasTraits, Float, Property, cached_property, \
    Instance, List, on_trait_change, Int, Tuple, Bool, \
    DelegatesTo, Event, Str, Button, Dict, Array, Any, \
    implements, File
    
from traitsui.api import \
    View, Item, Tabbed, VGroup, HGroup, ModelView, HSplit, VSplit, \
    CheckListEditor, EnumEditor, TableEditor, TabularEditor, Handler

from etsproxy.traits.ui.menu import Action, CloseAction, HelpAction, Menu, \
                                     MenuBar, NoButtons, Separator, ToolBar
                                         
from etsproxy.traits.ui.tabular_adapter \
    import TabularAdapter

from etsproxy.pyface.api import ImageResource
from .sim_todo import ToDo

from etsproxy.traits.ui.menu import OKButton
from .i_sim_model import ISimModel
from .i_sim_array import ISimArray
from .sim_array_view import SimArrayView

from etsproxy.traits.ui.file_dialog  \
    import open_file, save_file, FileInfo, TextInfo

import pickle
import os.path as path
    
from .sim_factor import \
    SimFactor, SimFloatFactor, SimIntFactor, SimEnumFactor
from .sim_output import SimOut
from .sim_array import SimArray

from numpy import array, linspace, frompyfunc, zeros, column_stack, \
                    log as ln, append, logspace, hstack, sign, trapz, mgrid, c_, \
                    zeros, arange, ix_

from math import exp, e, sqrt, log, pi
from scipy.special import erf, gamma
from scipy.stats import norm, weibull_min, uniform
from scipy.optimize import brentq, newton
from util.traits.editors.mpl_figure_editor import MPLFigureEditor
from matplotlib.figure import Figure
import string
from etsproxy.pyface.api import confirm, error, information, warning, YES, NO, CANCEL

TITLESTRING = 'simvisage.sim_array'

class SimPStudyController(Handler):

    def init( self, ui_info ):
        '''Set the title string
        '''
        self._set_title_string(ui_info, ignore_dirty = True )
    #---------------------------------------------------------------------------
    # Public Controller interface
    #---------------------------------------------------------------------------
    def new_study( self, ui_info ):

        if ui_info.object.dirty:

            # discard / save dialog
            
            answer = confirm( ui_info.ui.control, 'Study modified. Save it?',
                              title = 'New study',
                              cancel = False,
                              default = YES )

            if answer == YES:
                self.save_study( ui_info )

        ui_info.object.file_path = ''
        ui_info.object.new()
        self._set_title_string(ui_info)

    def exit_study(self, ui_info):
        
        if ui_info.object.dirty:

            # discard / save dialog
            answer = confirm( ui_info.ui.control, 'Save study before exiting?',
                              title = 'Close study',
                              cancel = True,
                              default = YES )

            if answer == YES:
                self.save_study( ui_info )
                self._on_close( ui_info )
                return True
            elif answer == NO:
                self._on_close( ui_info )
                return True
            else:
                return False
        else:
            self._on_close( ui_info )
            return True

#    def close(self, ui_info, is_ok ):
#        is_ok = self.exit_study( ui_info )
#        print 'IS OK', is_ok
#        super( SimPStudyController, self ).close( ui_info, is_ok )
#        self._on_close( ui_info )

    def open_study(self, ui_info):
        
        if ui_info.object.dirty:
            # discard / save dialog
            
            answer = confirm( ui_info.ui.control, 'Study modified. Save it?',
                              title = 'Open study',
                              cancel = True,
                              default = YES )

            if answer == YES:
                self.save_study( ui_info )
            elif answer == CANCEL:
                return

        file_name = open_file( filter = ['*.pst'], 
                               extensions = [FileInfo(), TextInfo()] )
        if file_name != '':
            ui_info.object.load( file_name )
            ui_info.object.file_path = file_name
            self._set_title_string(ui_info)
            
    def save_study(self, ui_info):
        
        if ui_info.object.file_path == '':
            file_name = save_file( filter = ['*.pst'], 
                                   extensions = [FileInfo(), TextInfo()] )
            if file_name == '':
                return
        else:
            file_name = ui_info.object.file_path
        
        ui_info.object.save( file_name )            

    def save_study_as(self, ui_info):
        
        file_name = save_file( filter = ['*.pst'], 
                               extensions = [FileInfo(), TextInfo()] )
        if file_name != '':
            ui_info.object.save( file_name )            
            ui_info.object.file_path = file_name
            self._set_title_string(ui_info)

    def new_view(self, ui_info):
        new_view = SimArrayView( model = ui_info.object.sim_array )
        new_view.configure_traits() #  kind = 'livemodal' )

    def exit_pstudy(self, ui_info):
        '''Close all views and check if everything was saved'''
        todo = ToDo()
        todo.configure_traits( kind = 'modal' )

    def clear_cache(self, ui_info):
        ui_info.object.clear_cache()

    def about_pstudy(self, ui_info):
        todo = ToDo()
        todo.configure_traits( kind = 'modal' )

    def _set_title_string(self, ui_info, ignore_dirty = False ):

        if ui_info.object.dirty and not ignore_dirty :
            modified = '(modified)'
        else:
            modified = ''
        if ui_info.object.file_path == '':
            filename = '<unnamed>'
        else:
            filename = ui_info.object.file_path
        
        title_string = '%s: %s %s' % (TITLESTRING, '<unnamed>', modified)
        ui_info.ui.title = title_string

class SimPStudy( HasTraits ):
    """ The main application window. """

    def __init__(self,**kw):
        super( SimPStudy, self ).__init__(**kw)

        # The initialization should not be considered dirty
        # therefore set the flag to indicate unsaved study to false 
        #
        self.dirty = False

    sim_array = Instance( SimArray )
    def _sim_array_default(self):
        return SimArray()

    sim_model = Property()
    def _set_sim_model(self, value):
        self.sim_array.sim_model = value

    def __getitem__( self, factor_slices ):
        '''Direct access to the sim_array.
        '''
        return self.sim_array[ factor_slices ]
    
    #---------------------------------------------------------------
    # PERSISTENCY
    #-------------------------------------------------------------------

    file_base_name = Property()
    def _get_file_base_name(self):
        return self.sim_model.__class__.__name__

    file_path = Str('')

    dirty = False
    @on_trait_change('sim_array.changed')
    def _set_dirty(self):
        self.dirty = True

    def new(self):
        sim_model = self.sim_array.sim_model
        self.sim_array = SimArray( sim_model = sim_model )
        self.dirty = False
    
    def load(self, file_name ):
        file = open( file_name, 'r' )            
        self.sim_array = pickle.load(file)
        file.close()
        self.dirty = False
        
    def save(self, file_name ):
        file = open( file_name, 'w' )
        pickle.dump( self.sim_array, file )
        file.close()
        self.dirty = False

    def clear_cache(self):
        self.sim_array.clear_cache()
        
    toolbar = ToolBar( 
                  Action(name="New Study",
                         tooltip = 'Create a new study',
                         image = ImageResource('New-32'),
                         action="new_study"),
                  Action(name="Open Study",
                         tooltip = 'Open a study',
                         image = ImageResource('fileopen-32'),
                         action="open_study"),
                  Action(name="Save Study",
                         tooltip = 'Save study',
                         image = ImageResource('save'),
                         action="save_study"),
                  Action(name="New View",
                         tooltip = 'Create new view',
                         image = ImageResource('new_view'),
                         action="new_view"),
                  Action(name="Clear Cache",
                         tooltip = 'Reset cache',
                         image = ImageResource('reset'),
                         action="clear_cache"),
                  image_size = (22,22),
                  show_tool_names = False,
                  show_divider = True,
                  name = 'study_toolbar' )                                         

    menubar = MenuBar(Menu(Action(name="&New",
                               action="new_study"),
                            Action(name="&Open",
                               action="open_study"),
                            Action(name="&Save",
                               action="save_study"),
                            Action(name="Save &As",
                               action="save_study_as"),
                            Action(name="&Exit",
                               action="exit_study"),
                            name="&File"),
                      Menu(Action(name="&New View",
                            action="new_view"),
                            name="&View"),                                         
                      Menu(Action(name="&Clear Cache",
                            action="clear_cache"),
                            name="&Data"),                                          
                    Menu(Action(name="About PStudy",
                               action="about_pstudy"),
                         HelpAction,                                                   
                        name="Help")                                            
                    )
    
    view = View(
                Item('sim_array@', show_label = False ),
                id = 'simvisage.simiter.pstudy',
                dock = 'tab',
                menubar = menubar, 
                toolbar = toolbar,
                resizable = True,
                width = 0.8,
                height = 0.8,
                title = 'SimVisage: Parametric Study',
                handler = SimPStudyController,
                )
    
def run():
    
    from .sim_model import SimModel
    pstudy_app = SimPStudy( sim_model = SimModel() )
    pstudy_app.configure_traits( kind = 'live' )
    
if __name__ == '__main__':
    run()                     


# -------------------------------------------------------------------------
#
# Copyright (c) 2009, IMB, RWTH Aachen.
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in simvisage/LICENSE.txt and may be redistributed only
# under the conditions described in the aforementioned license.  The license
# is also available online at http://www.simvisage.com/licenses/BSD.txt
#
# Thanks for using Simvisage open source!
#
# Created on Feb 23, 2010 by: rch

from traits.api import \
    List, Int, Float, \
    on_trait_change, Instance, \
    Array, Property, cached_property, \
    Bool, Enum, Event

from traitsui.api import \
    View, Item, HSplit, VGroup, \
    TableEditor, VSplit, Group, \
    Spring

from traitsui.table_column import \
    ObjectColumn

from numpy import \
    array, ones_like, argmax

import numpy as np

from .concrete_mixture \
    import ConcreteMixture

from .fabric_layup \
    import FabricLayUp

from matresdev.db.simdb.simdb_class import \
    SimDBClass

from .ccs_unit_cell import \
    CCSUnitCell

ccs_table_editor = TableEditor(
    columns=[ObjectColumn(name='fabric_layout_key',
                          editable=True, width=0.20),
             ObjectColumn(name='n_layers',
                          editable=True, width=0.20),
             ObjectColumn(name='s_tex_z',
                          editable=True, width=0.20),
             ObjectColumn(name='orientation_fn_key',
                          editable=True, width=0.20),
             ObjectColumn(name='n_layers_0',
                          editable=False, width=0.20),
             ObjectColumn(name='n_layers_90',
                          editable=False, width=0.20),
             ObjectColumn(name='thickness',
                          editable=False, width=0.20),
             ObjectColumn(name='is_regular',
                          editable=False, width=0.14), ],
    selection_mode='row',
    selected='object.selected_layup',
    show_toolbar=True,
    row_factory=FabricLayUp,
    editable=True,
    auto_add=False,
    configurable=True,
    sortable=True,
    deletable=True,
    reorderable=True,
    sort_model=False,
    auto_size=False,
    #                        filters      = [EvalFilterTemplate,
    #                                       MenuFilterTemplate,
    #                                       RuleFilterTemplate ],
    #                        search       = EvalTableFilter(),
)


class CompositeCrossSection(SimDBClass):

    '''Describes the combination of the concrete mixture and textile
    cross section.
    Note: only one concrete can be defined for the entire composite
    cross section.
    '''

    input_change = Event

    @on_trait_change('+input,fabric_layup_list.input_change')
    def _set_input_change(self):
        self.input_change = True

    # -------------------------------------------------------------------------
    # select the concrete mixture from the concrete database:
    # -------------------------------------------------------------------------

    concrete_mixture_key = Enum(list(ConcreteMixture.db.keys()),
                                simdb=True, input=True,
                                auto_set=False, enter_set=True)

    concrete_mixture_ref = Property(Instance(SimDBClass),
                                    depends_on='concrete_mixture_key')

    @cached_property
    def _get_concrete_mixture_ref(self):
        return ConcreteMixture.db[self.concrete_mixture_key]

    # -------------------------------------------------------------------------
    # define the composite cross section as a list of fabric layups:
    # -------------------------------------------------------------------------

    fabric_layup_list = List(Instance(FabricLayUp), input=True)

    # default layup necessary to show view the first time.
    def _fabric_layup_list_default(self):
        return [plain_concrete(1.0)]

    selected_layup = Instance(FabricLayUp, transient=True)

    def _selected_layup_default(self):
        return self.fabric_layup_list[0]

    # -------------------------------------------------------------------------
    # calculated material properties for the composite
    # -------------------------------------------------------------------------

    thickness_arr = Property(Array(Float, unit='m'), depends_on='input_change')

    @cached_property
    def _get_thickness_arr(self):
        '''thickness of each fabric layup.
        '''
        return array([flu.thickness for flu in self.fabric_layup_list],
                     dtype='float_')

    thickness = Property(Float, unit='m', depends_on='input_change')

    @cached_property
    def _get_thickness(self):
        '''thickness of the entire composite cross section.
        '''
        return sum(self.thickness_arr)

    a_tex_arr = Property(Array(Float, unit='m'), depends_on='input_change')

    @cached_property
    def _get_a_tex_arr(self):
        '''textile cross section of each fabric layup.
        '''
        return array([flu.a_tex for flu in self.fabric_layup_list],
                     dtype='float_')

    a_tex = Property(Float, unit='mm^2/m', depends_on='input_change')

    @cached_property
    def _get_a_tex(self):
        '''textile cross section of the entire cross section per meter width.
        '''
        return sum(self.a_tex_arr)

    rho_c = Property(Float, unit='-', depends_on='input_change')

    @cached_property
    def _get_rho_c(self):
        '''reinforcement ratio of the composite material.
        '''
        return self.a_tex / self.thickness / 1000000  # [-]

    rho_arr = Property(Array(Float, unit='-'), depends_on='input_change')

    @cached_property
    def _get_rho_arr(self):
        '''reinforcement ration of each fabric layup.
        '''
        return array([flu.rho for flu in self.fabric_layup_list],
                     dtype='float_')

    E_tex_arr = Property(Array(Float, unit='MPa'), depends_on='input_change')

    @cached_property
    def _get_E_tex_arr(self):
        '''textile E-modulus of each fabric layup.
        '''
        return array([flu.E_tex for flu in self.fabric_layup_list],
                     dtype='float_')

    E_tex = Property(Float, unit='MPa', depends_on='input_change')

    @cached_property
    def _get_E_tex(self):
        '''get the larges value of the textile reinforcement
        for the plot of the analytical stiffness
        in 'exp_browse'
        '''
        return np.max(self.E_tex_arr, axis=0)
        # smeared textile E-modulus of the entire composite.
#        return sum( self.E_tex_arr * self.thickness_arr ) / self.thickness

    def get_E_m_time(self, age):
        '''function for the concrete matrix E-modulus
        as function depending of the concrete age.
        '''
        E_m = self.concrete_mixture_ref.get_E_m_time(age)
        return E_m

    def get_E_c_time(self, age):
        '''function for the composite E-modulus as weighted sum of all
        fabric layups. Returns a function depending of the concrete age.
        '''
        rho_arr = self.rho_arr
        E_tex_arr = self.E_tex_arr
        E_m = self.concrete_mixture_ref.get_E_m_time(age)
        E_m_arr = E_m * ones_like(rho_arr)
        return sum(((1 - rho_arr) * E_m_arr + rho_arr * E_tex_arr) *
                   self.thickness_arr) / self.thickness

    E_c28 = Property(Float, unit='MPa', depends_on='input_change')

    @cached_property
    def _get_E_c28(self):
        '''Composite E-modulus after 28 days.
        '''
        return self.get_E_c_time(28)

    max_rho_idx = Property(Int, unit='-', depends_on='input_change')

    @cached_property
    def _get_max_rho_idx(self):
        '''return the index of the dominating layup in 'flu_list'.
        '''
        return argmax(self.rho_arr)

    max_rho = Property(Float, unit='m', depends_on='input_change')

    @cached_property
    def _get_max_rho(self):
        '''return the reinforcement ratio of the dominating layup in 'flu_list'.
        '''
        return self.rho_arr[self.max_rho_idx]

    is_regular = Property(Bool)

    def _get_is_regular(self):
        # first check to see if this calibration is acceptable at all.
        # there must be only one layup with unreinforced coverage
        # to derive material parameters that can be reused for non-constant
        # cross sectional stress profiles
        #
        if len(self.fabric_layup_list) > 3:
            return False

        # get the proportion of the reinforcement ratio
        # of the other layers
        #
        flu_rho_fraction = self.max_rho / self.rho_c
        if (flu_rho_fraction) < 0.90:
            return False

        return True

    # -------------------------------------------------------------------------
    # Calibration parameters
    # -------------------------------------------------------------------------
    def set_param(self, material_model, calibration_test, df):

        if not self.is_regular:
            raise ValueError('Too complex cross section for calibration\n' \
                'There must be one dominating layup with a 90% contribution' \
                'to the reinforcement ration of the composite cross section')

        # Extract the characteristics of the dominating layout
        cm_key = self.concrete_mixture_key

        flu = self.fabric_layup_list[self.max_rho_idx]
        orientation_fn_key = flu.orientation_fn_key

        flo_key = flu.fabric_layout_key
        s_tex_z = flu.s_tex_z

        # construct the key of the composite cross section unit cell
        # and store it
        #
        key = cm_key + '_' + flo_key + '_' + \
            '%.5f' % (s_tex_z) + '_' + orientation_fn_key

        ccsuc = CCSUnitCell.db.get(key, None)
        if ccsuc is None:
            ccsuc = CCSUnitCell(concrete_mixture_key=cm_key,
                                fabric_layout_key=flo_key,
                                s_tex_z=s_tex_z,
                                orientation_fn_key=orientation_fn_key,
                                rho=self.rho_c)
            CCSUnitCell.db[key] = ccsuc

        print('CCSUnitCell: stored parameters for', key)
        ccsuc.set_param(material_model, calibration_test, df)
        CCSUnitCell.db[key].save()

    # -------------------------------------------------------------------------
    # view
    # -------------------------------------------------------------------------

    traits_view = View(Item('key', style='readonly', show_label=False),
                       VSplit(
        VGroup(
            Spring(),
            Item(
                'concrete_mixture_key', label='concrete mixture'),
            Spring(),
            Item('concrete_mixture_ref@', show_label=False),
            Spring(),
            id='exdb.ccs.cm',
            label='concrete mixture',
            dock='tab',
        ),
        HSplit(
            Item('fabric_layup_list@', editor=ccs_table_editor,
                 show_label=False, resizable=True),
            Item('selected_layup@',
                 show_label=False, resizable=True),
            # label = 'fabric layup',
            # id = 'exdb.ccs.flu',
            #                              scrollable = True,
            #                              dock = 'tab',
        ),
        Group(
            Item('thickness',
                 style='readonly', show_label=True, format_str="%.3f"),
            Item('rho_c',
                 style='readonly', show_label=True, format_str="%.4f"),
            Item('E_c28',
                 style='readonly', show_label=True, format_str="%.0f"),
            Item('is_regular',
                 style='readonly', show_label=True),
            label='derived params',
            id='exdb.ccs.dp',
            dock='tab',
        ),

        dock='tab',
        id='ccs.db.vsplit',
        orientation='vertical',
    ),
        dock='tab',
        id='ccs.db',
        scrollable=True,
        resizable=True,
        height=0.4,
        width=0.5,
        buttons=['OK', 'Cancel'],
    )


# convenience definition of a layup consisting of pure concrete
#
def plain_concrete(thickness):
    return FabricLayUp(
        n_layers=1,
        s_tex_z=thickness,
        fabric_layout_key='unreinforced',
        orientation_fn_key='unreinforced'
    )

# --------------------------------------------------------------------------

if __name__ == '__main__':

    s_tex_z = 0.030 / (7 + 1)
    ccs = CompositeCrossSection(
        fabric_layup_list=[
            plain_concrete(s_tex_z * 0.5),
            FabricLayUp(
                n_layers=7,
                orientation_fn_key='unreinforced',
                s_tex_z=s_tex_z,
                fabric_layout_key='MAG-07-03'
            ),
            plain_concrete(s_tex_z * 0.5)
        ],
        concrete_mixture_key='PZ-0708-1'
    )

    ccs.configure_traits()



'''
Created on 26. 6. 2014

@author: Vancikv
'''

from traits.api import \
    Instance, Button, Str

from traitsui.api import \
    View, Item, HGroup, Handler, spring, UIInfo

from traitsui.menu import \
    Action

class MxNClassExtHandler(Handler):
    '''Handles adding and removing of ReinfFabric database
    objects through user interface.
    '''

    # The UIInfo object associated with the view:
    info = Instance(UIInfo)

    ok = Button('OK')
    delete = Button('OK')
    cancel = Button('Cancel')

    material_name = Str

    # The pop-up customization view:
    new_view = View(
        HGroup(
            spring,
            Item('material_name'),
            Item('ok', show_label=False),
            Item('cancel', show_label=False),
        ),
        kind='popup'
    )

    del_view = View(
        HGroup(
            spring,
            Item(name='', label='Really delete?'),
            Item('delete', show_label=False),
            Item('cancel', show_label=False),
        ),
        kind='popup'
    )

    # Event handlers:

    def object_new_material_changed (self, info):
        if info.initialized:
            self.info = info
            self._ui = self.edit_traits(parent=info.new_material.control, view='new_view')

    def object_del_material_changed (self, info):
        if info.initialized:
            self.info = info
            self._ui = self.edit_traits(parent=info.del_material.control, view='del_view')

    def _ok_fired (self):
        object = self.info.object
        if self.material_name == '':
            print('Please enter material name!')
        elif object.get(self.material_name, None):
            print('Material name already occupied!')
        else:
            object[self.material_name] = object.klass()
            self._ui.dispose()
            self.info.ui.parent.updated = True

    def _delete_fired (self):
        object = self.info.object
        del object[object.chosen_instance]
        self._ui.dispose()
        self.info.ui.parent.updated = True

    def _cancel_fired (self):
        self._ui.dispose()



'''
Created on Aug 23, 2012

@author: rch
'''

from bmcs_beam.mxn.constitutive_law import CLBase
from bmcs_beam.mxn.mxn_class_extension import \
    MxNClassExt
from bmcs_beam.mxn.mxn_tree_node import \
    MxNLeafNode
from traits.api import \
    Float, Property, cached_property

from bmcs_beam.mxn.matresdev.db.simdb import \
    SimDBClass, SimDBClassExt
import numpy as np



class MatrixLawBase(CLBase, MxNLeafNode, SimDBClass):

    '''Base class for concrete constitutive laws.'''
    # characteristic compressive stress [MPa]
    #
    f_ck = Float(60.0, simdb=True, enter_set=True, auto_set=False, input=True)
    eps_c_u = Float(
        0.0033, simdb=True, enter_set=True, auto_set=False, input=True)
    E_c = Float(28e+3, simdb=True, enter_set=True, auto_set=False, input=True)
    node_name = 'Constitutive law'

    high_strength_level = Float(
        50.0, simdb=True, enter_set=True, auto_set=False, input=True)

    eps_arr = Property(depends_on='+input')

    @cached_property
    def _get_eps_arr(self):
        return self.mfn.xdata

    sig_arr = Property(depends_on='+input')

    @cached_property
    def _get_sig_arr(self):
        return self.mfn.ydata

MatrixLawBase.db = MxNClassExt(
    klass=MatrixLawBase,
    verbose='io',
    node_name='Matrix law database'
)

if __name__ == '__main__':
    MatrixLawBase.db.configure_traits()



#!/usr/bin/env python
""" The entry point for an Envisage application. """

# Standard library imports.
import sys
import os.path
import logging

# Enthought library imports.
#from etsproxy.mayavi.plugins.app import get_plugins, setup_logger
from etsproxy.mayavi.plugins.app import setup_logger
from traits.api import List, Instance
from etsproxy.envisage.api import Plugin, ServiceOffer, ExtensionPoint
from etsproxy.pyface.workbench.api import Perspective, PerspectiveItem

logger = logging.getLogger()

# View IDs.
PROMOD_VIEW = 'ibvpy.plugins.promod_service.promod_service'
TSTEPPER_VIEW = 'ibvpy.plugins.tstepper_service.tstepper_service' 
TLOOP_VIEW = 'ibvpy.plugins.tloop_service.tloop_service' 
RTRACEMNGR_VIEW = 'ibvpy.plugins.rtrace_service.rtrace_service' 

###############################################################################
# `IBVPPerspective` class.
###############################################################################
class IBVModelSpecifyPerspective(Perspective):
    """ An default perspective for the app. """

    # The perspective's name.
    name = 'Specify IBV Model'

    # Should this perspective be enabled or not?
    enabled = True

    # Should the editor area be shown in this perspective?
    show_editor_area = True

    # The contents of the perspective.
    contents = [
        PerspectiveItem(id = PROMOD_VIEW, position = 'top'),
        PerspectiveItem(id = TSTEPPER_VIEW, position = 'bottom'),
        ]

###############################################################################
# `ProModPlugin` class.
###############################################################################
class ProModUIPlugin(Plugin):

    # Extension points we contribute to.
    PERSPECTIVES = 'etsproxy.envisage.ui.workbench.perspectives'
    VIEWS = 'etsproxy.envisage.ui.workbench.views'

    # The plugin's unique identifier.
    id = 'promod_service.promod_service'

    # The plugin's name (suitable for displaying to the user).
    name = 'Product Model'

    # Perspectives.
    perspectives = List(contributes_to = PERSPECTIVES)

    # Views.
    views = List(contributes_to = VIEWS)

    ######################################################################
    # Private methods.
    def _perspectives_default(self):
        """ Trait initializer. """
        return [ProModSpecifyPerspective, ProModAnalyzePerspective]

    def _views_default(self):
        """ Trait initializer. """
        return [self._promod_service_view_factory]

    def _promod_service_view_factory(self, window, **traits):
        """ Factory method for promod_service views. """
        from etsproxy.pyface.workbench.traits_ui_view import \
                TraitsUIView

        promod_service = self._get_promod_service(window)
        tui_engine_view = TraitsUIView(obj = promod_service,
                                       id = 'promod.plugins.promod_service.promod_service',
                                       name = 'Product Model',
                                       window = window,
                                       position = 'left',
                                       **traits
                                       )
        return tui_engine_view

    def _get_promod_service(self, window):
        """Return the promod_service service."""
        return window.get_service('matresdev.db.plugins.promod_service.ProModService')
